@@ -32,6 +32,7 @@
 
 using namespace std;
 
+//#include "rtty.h"
 #include "view_rtty.h"
 #include "fl_digi.h"
 #include "digiscope.h"
@@ -47,9 +48,11 @@ using namespace std;
 #include "main.h"
 #include "modem.h"
 
+#include "rtty.h"
+
 #define FILTER_DEBUG 0
 
-view_rtty *rttyviewer = (view_rtty *)0;
+#define SHAPER_BAUD 150
 
 //=====================================================================
 // Baudot support
@@ -76,10 +79,13 @@ int dspcnt = 0;
 
 static char msg1[20];
 
-const double rtty::SHIFT[] = {23, 85, 160, 170, 182, 200, 240, 350, 425, 850};
-const double rtty::BAUD[]  = {45, 45.45, 50, 56, 75, 100, 110, 150, 200, 300};
-const int	rtty::BITS[]  = {5, 7, 8};
-const int	rtty::numshifts = (int)(sizeof(SHIFT) / sizeof(*SHIFT));
+const double	rtty::SHIFT[] = {23, 85, 160, 170, 182, 200, 240, 350, 425, 850};
+// FILTLEN must be same size as BAUD
+const double	rtty::BAUD[]  = {45, 45.45, 50, 56, 75, 100, 110, 150, 200, 300};
+const int		rtty::FILTLEN[] = { 512, 512, 512, 512, 512, 512, 512, 256, 128, 64};
+const int		rtty::BITS[]  = {5, 7, 8};
+const int		rtty::numshifts = (int)(sizeof(SHIFT) / sizeof(*SHIFT));
+const int		rtty::numbauds = (int)(sizeof(BAUD) / sizeof(*BAUD));
 
 void rtty::tx_init(SoundBase *sc)
 {
@@ -181,6 +187,8 @@ void rtty::init()
 
 rtty::~rtty()
 {
+	if (rttyviewer) delete rttyviewer;
+
 	if (mark_filt) delete mark_filt;
 	if (space_filt) delete space_filt;
 	if (pipe) delete [] pipe;
@@ -194,21 +202,12 @@ rtty::~rtty()
 
 void rtty::reset_filters()
 {
-	int filter_length = 1024;
-
-	if (mark_filt) {
-		mark_filt->rtty_filter(rtty_baud/samplerate);
-	} else {
-		mark_filt = new fftfilt(rtty_baud/samplerate, filter_length);
-		mark_filt->rtty_filter(rtty_baud/samplerate);
-     }
-
-	if (space_filt) {
-		space_filt->rtty_filter(rtty_baud/samplerate);
-	} else {
-		space_filt = new fftfilt(rtty_baud/samplerate, filter_length);
-		space_filt->rtty_filter(rtty_baud/samplerate);
-     }
+	delete mark_filt;
+	mark_filt = new fftfilt(rtty_baud/samplerate, filter_length);
+	mark_filt->rtty_filter(rtty_baud/samplerate);
+	delete space_filt;
+	space_filt = new fftfilt(rtty_baud/samplerate, filter_length);
+	space_filt->rtty_filter(rtty_baud/samplerate);
 }
 
 void rtty::restart()
@@ -217,8 +216,10 @@ void rtty::restart()
 
 	rtty_shift = shift = (progdefaults.rtty_shift < numshifts ?
 				  SHIFT[progdefaults.rtty_shift] : progdefaults.rtty_custom_shift);
-printf("rtty_shift %.1f\n", shift);
+	if (progdefaults.rtty_baud > numbauds - 1) progdefaults.rtty_baud = numbauds - 1;
 	rtty_baud = BAUD[progdefaults.rtty_baud];
+	filter_length = FILTLEN[progdefaults.rtty_baud];
+
 	nbits = rtty_bits = BITS[progdefaults.rtty_bits];
 	if (rtty_bits == 5)
 		rtty_parity = RTTY_PARITY_NONE;
@@ -296,7 +297,9 @@ printf("rtty_shift %.1f\n", shift);
 
 	for (int i = 0; i < MAXPIPE; i++) mark_history[i] = space_history[i] = cmplx(0,0);
 
-	rttyviewer->restart();
+//	if (::rttyviewer) ::rttyviewer->restart();
+	if (rttyviewer) rttyviewer->restart();
+
 	progStatus.rtty_filter_changed = false;
 
 }
@@ -317,7 +320,9 @@ rtty::rtty(trx_mode tty_mode)
 	pipe = new double[MAXPIPE];
 	dsppipe = new double [MAXPIPE];
 
-	::rttyviewer = new view_rtty(mode);
+//	if (::rttyviewer == 0) ::rttyviewer = new view_rtty(mode);
+
+	rttyviewer = new view_rtty(mode);
 
 	m_Osc1 = new Oscillator( samplerate );
 	m_Osc2 = new Oscillator( samplerate );
@@ -609,7 +614,7 @@ double value;
 if (snum < 2 * filter_length) {
 	frequency = 1000.0;
 	ook(snum);
-	z = complex(value, value);
+	z = cmplx(value, value);
 	ook_signal << snum << "," << z.real() << ",";
 //	snum++;
 } else {
@@ -674,6 +679,7 @@ if (mnum < 2 * filter_length)
 
 //			double v0, v1, v2, v3, v4, v5;
 			double v3;
+
 // no ATC
 //			v0 = mark_mag - space_mag;
 // Linear ATC
@@ -696,7 +702,6 @@ if (mnum < 2 * filter_length)
 //					(sclipped - noise_floor) * (sclipped - noise_floor) - 0.25 * (
 //					(mark_env - noise_floor) * (mark_env - noise_floor) -
 //					(space_env - noise_floor) * (space_env - noise_floor));
-
 //				switch (progdefaults.rtty_demodulator) {
 //			switch (2) { // Optimal ATC
 //			case 0: // linear ATC
@@ -862,11 +867,13 @@ double rtty::FSKnco()
 void rtty::send_symbol(int symbol, int len)
 {
 	acc_symbols += len;
-#if 0
+
+//#if !SHAPER_BAUD
+if (!progStatus.shaped_rtty) {
+//if (rtty_baud > SHAPER_BAUD) {
 	double freq;
 
-	if (reverse)
-		symbol = !symbol;
+	if (reverse) symbol = !symbol;
 
 	if (symbol)
 		freq = get_txfreq_woffset() + shift / 2.0;
@@ -880,8 +887,8 @@ void rtty::send_symbol(int symbol, int len)
 		else
 			FSKbuf[i] = 0.0 * FSKnco();
 	}
-
-#else
+} else {
+//#else
 
 	double const freq1 = get_txfreq_woffset() + shift / 2.0;
 	double const freq2 = get_txfreq_woffset() - shift / 2.0;
@@ -900,6 +907,7 @@ void rtty::send_symbol(int symbol, int len)
 				mark  = m_SymShaper1->Update( sym) * m_Osc1->Update( freq1 );
 				space = m_SymShaper2->Update(!sym) * m_Osc2->Update( freq2 );
 				signal = mark + space;
+
 				if (maxamp < fabs(signal)) maxamp = fabs(signal);
 			}
 		}
@@ -908,8 +916,8 @@ void rtty::send_symbol(int symbol, int len)
 	for( int i = 0; i < len; ++i ) {
 		mark  = m_SymShaper1->Update( symbol) * m_Osc1->Update( freq1 );
 		space = m_SymShaper2->Update(!symbol) * m_Osc2->Update( freq2 );
-
 		signal = mark + space;
+
 		if (maxamp < fabs(signal)) {
 			maxamp = fabs(signal);
 		}
@@ -920,8 +928,8 @@ void rtty::send_symbol(int symbol, int len)
 		else
 			FSKbuf[i] = 0.0 * FSKnco();
 	}
-#endif
-
+}
+//#endif
 	if (progdefaults.PseudoFSK)
 		ModulateStereo(outbuf, FSKbuf, symbollen);
 	else
@@ -930,7 +938,9 @@ void rtty::send_symbol(int symbol, int len)
 
 void rtty::send_stop()
 {
-#if 0
+//#if !SHAPER_BAUD
+if (!progStatus.shaped_rtty) {
+//if (rtty_baud >= SHAPER_BAUD) {
 	double freq;
 	bool invert = reverse;
 
@@ -946,7 +956,8 @@ void rtty::send_stop()
 		else
 			FSKbuf[i] = FSKnco();
 	}
-#else
+} else {
+//#else
 
 	double const freq1 = get_txfreq_woffset() + shift / 2.0;
 	double const freq2 = get_txfreq_woffset() - shift / 2.0;
@@ -960,8 +971,8 @@ void rtty::send_stop()
 	for( int i = 0; i < stoplen; ++i ) {
 		mark  = m_SymShaper1->Update( symbol)*m_Osc1->Update( freq1 );
 		space = m_SymShaper2->Update(!symbol)*m_Osc2->Update( freq2 );
-
 		signal = mark + space;
+
 		if (maxamp < fabs(signal)) maxamp = fabs(signal);
 		outbuf[i] = maxamp ? (0.99 * signal / maxamp) : 0.0;
 
@@ -970,13 +981,12 @@ void rtty::send_stop()
 		else
 			FSKbuf[i] = FSKnco();
 	}
-#endif
-
+}
+//#endif
 	if (progdefaults.PseudoFSK)
 		ModulateStereo(outbuf, FSKbuf, stoplen);
 	else
 		ModulateXmtr(outbuf, stoplen);
-
 }
 
 void rtty::flush_stream()
@@ -988,8 +998,8 @@ void rtty::flush_stream()
 	for( int i = 0; i < symbollen * 6; ++i ) {
 		mark  = m_SymShaper1->Update(0)*m_Osc1->Update( freq1 );
 		space = m_SymShaper2->Update(0)*m_Osc2->Update( freq2 );
-
 		signal = mark + space;
+
 		if (maxamp < fabs(signal)) maxamp = fabs(signal);
 		outbuf[i] = maxamp ? (0.99 * signal / maxamp) : 0.0;
 
@@ -1036,6 +1046,8 @@ void rtty::send_char(int c)
 		if (c)
 			put_echo_char(progdefaults.rx_lowercase ? tolower(c) : c);
 	}
+	else
+		put_echo_char(c);
 }
 
 void rtty::send_idle()
@@ -1079,10 +1091,11 @@ int rtty::tx_process()
 			send_char(0x08);
 			send_char(0x02);
 		}
-#if 1
-// if (progdefaults.rtty_shaper)
-		flush_stream();
-#endif
+	if (progStatus.shaped_rtty) flush_stream();
+//	if (rtty_baud <= SHAPER_BAUD) flush_stream();
+//#if SHAPER_BAUD
+//	flush_stream();
+//#endif
 		cwid();
 		return -1;
 	}
@@ -1100,11 +1113,6 @@ int rtty::tx_process()
 		send_char(c);
 		xmt_samples = char_samples = acc_symbols;
 
-//		printf("%5s %d samples, overhead %d, %f sec's\n",
-//			ascii3[c & 0xff],
-//			char_samples,
-//			ovhd_samples,
-//			1.0 * char_samples / samplerate);
 		return 0;
 	}
 
@@ -1167,12 +1175,6 @@ int rtty::tx_process()
 	send_char(c & 0x1F);
 	xmt_samples = char_samples = acc_symbols;
 
-//	printf("%5s %d samples, overhead %d, %f sec's\n",
-//		ascii3[c & 0xff],
-//		char_samples,
-//		ovhd_samples,
-//		1.0 * char_samples / samplerate);
-
 	return 0;
 }
 
@@ -1237,7 +1239,7 @@ Oscillator::Oscillator( double samplerate )
 {
 	m_phase = 0;
 	m_samplerate = samplerate;
-	std::cerr << "samplerate for Oscillator:"<<m_samplerate<<"\n";
+//	std::cerr << "samplerate for Oscillator:"<<m_samplerate<<"\n";
 }
 
 double Oscillator::Update( double frequency )
@@ -1301,8 +1303,11 @@ void SymbolShaper::Preset(double baud, double sr)
 
     for( int x=0; x<m_table_size; ++x ) {
         int const offset = m_table_size/2;
-        double const T = sample_rate / (baud_rate*2.0); // symbol-length in samples
-        double const t = (x-offset); // symbol-time relative to zero
+        double wfactor = 1.0 / 1.568; // optimal
+// symbol-length in samples if wmultiple = 1.0
+        double const T = wfactor * sample_rate / (baud_rate*2.0); 
+// symbol-time relative to zero
+        double const t = (x-offset);
 
         m_sinc_table[x] = rcos( t, T, 1.0 );
 
