@@ -58,12 +58,13 @@ using namespace std;
 
 void  clear_xmt_arrays();
 
-static int symlen = 4096; // nominal symbol length; 3 baud
-#define SQLFILT_SIZE 200
-#define NIT std::string::npos
+#define SQLFILT_SIZE 64
 
+#define NIT std::string::npos
 #define txcenterfreq  1500.0
 
+int fsq::symlen = 4096; // nominal symbol length; 3 baud
+
 static const char *FSQBOL = " \n";
 static const char *FSQEOL = "\n ";
 static const char *FSQEOT = "  \b  ";
@@ -87,8 +88,8 @@ static std::string sz2utf8(std::string s)
 	return s;
 }
 
-static int nibbles[199];
-static void init_nibbles()
+//static int nibbles[199];
+void fsq::init_nibbles()
 {
 	int nibble = 0;
 	for (int i = 0; i < 199; i++) {
@@ -148,7 +149,11 @@ fsq::fsq(trx_mode md) : modem()
 	samplerate = SR;
 	fft = new g_fft<double>(FFTSIZE);
 	snfilt = new Cmovavg(SQLFILT_SIZE);
-	baudfilt = new Cmovavg(3);
+
+	noisefilt = new Cmovavg(32);
+	sigfilt = new Cmovavg(8);
+
+//	baudfilt = new Cmovavg(3);
 	movavg_size = progdefaults.fsq_movavg;
 	if (movavg_size < 1) movavg_size = progdefaults.fsq_movavg = 1;
 	if (movavg_size > 4) movavg_size = progdefaults.fsq_movavg = 4;
@@ -206,9 +211,13 @@ fsq::~fsq()
 {
 	delete fft;
 	delete snfilt;
+
+	delete sigfilt;
+	delete noisefilt;
+
 	for (int i = 0; i < NUMBINS; i++)
 		delete binfilt[i];
-	delete baudfilt;
+//	delete baudfilt;
 	delete picfilter;
 	REQ(close_fsqMonitor);
 	stop_sounder();
@@ -363,6 +372,7 @@ int fsq::valid_callsign(std::string s)
 	if (s == allcall) return 2;
 	if (s == cqcqcq) return 4;
 	if (s == mycall) return 1;
+	if (s.find("Heard") != string::npos) return 0;
 
 	static char sz[21];
 	memset(sz, 0, 21);
@@ -423,7 +433,6 @@ void fsq::parse_rx_text()
 	if (station_calling == mycall) { // do not display any of own rx stream
 		LOG_ERROR("Station calling is mycall: %s", station_calling.c_str());
 		rx_text.erase(0, p+3);
-//		rx_text.clear();
 		return;
 	}
 	if (!station_calling.empty()) {
@@ -448,10 +457,13 @@ void fsq::parse_rx_text()
 	size_t tr_pos = 0;
 	char tr = rx_text[tr_pos];
 	size_t trigger = triggers.find(tr);
+
 // strip any leading spaces before either text or first directed callsign
+
 	while (rx_text.length() > 1 &&
 		triggers.find(rx_text[0]) != std::string::npos)
 		rx_text.erase(0,1);
+
 // find first word
 	while ( tr_pos < rx_text.length()
 			&& ((trigger = triggers.find(rx_text[tr_pos])) == std::string::npos) ) {
@@ -465,18 +477,22 @@ void fsq::parse_rx_text()
 			rx_text.insert(0," ");
 			break; // not a callsign
 		}
-		if (word_is == 1) directed = true; // mycall
+		if (word_is == 1) {
+			directed = true; // mycall
+		}
 		// test for cqcqcq and allcall
-		else if (word_is != 8) all = true;
+		else if (word_is != 8)
+			all = true;
 
 		rx_text.erase(0, tr_pos);
-		if (rx_text[0] != ' ') break;
 
 		while (rx_text.length() > 1 &&
 			(rx_text[0] == ' ' && rx_text[1] == ' '))
 			rx_text.erase(0,1);
 
+		if (rx_text[0] != ' ') break;
 		rx_text.erase(0, 1);
+
 		tr_pos = 0;
 		tr = rx_text[tr_pos];
 		trigger = triggers.find(tr);
@@ -799,9 +815,6 @@ void fsq::parse_carat(std::string relay)
 	display_fsq_rx_text(toprint.append(rx_text).append("\n"), FTextBase::FSQ_DIR);
 }
 
-double maxfreq = 0;
-double minfreq = 4000;
-
 void fsq::parse_pcnt()
 {
 	switch (rx_text[2]) {
@@ -832,8 +845,6 @@ void fsq::parse_pcnt()
 
 	}
 	REQ( fsq_showRxViewer, picW, picH, rx_text[2] );
-	maxfreq = 0;
-	minfreq = 4000;
 
 	image_counter = 0;
 
@@ -991,8 +1002,6 @@ void fsq::process_symbol(int sym)
 			lf_check(curr_ch);
 
 			if (b_bot) {
-				double val = snfilt->value();
-				for (int i = 0; i < SQLFILT_SIZE; i++) snfilt->run(val);
 				ch_sqlch_open = true;
 				rx_text.clear();
 			}
@@ -1003,12 +1012,18 @@ void fsq::process_symbol(int sym)
 					display_fsq_mon_text( fsq_bot, FTextBase::CTRL);
 				if (b_eol) {
 					display_fsq_mon_text( fsq_eol, FTextBase::CTRL);
-					for (int i = 0; i < SQLFILT_SIZE; i++) snfilt->run(0);
+					noisefilt->reset();
+					noisefilt->run(1);
+					sigfilt->reset();
+					sigfilt->run(1);
 					snprintf(szestimate, sizeof(szestimate), "%.0f db", s2n );
 				}
 				if (b_eot) {
 					snprintf(szestimate, sizeof(szestimate), "%.0f db", s2n );
-					for (int i = 0; i < SQLFILT_SIZE; i++) snfilt->run(0);
+					noisefilt->reset();
+					noisefilt->run(1);
+					sigfilt->reset();
+					sigfilt->run(1);
 					display_fsq_mon_text( fsq_eot, FTextBase::CTRL);
 				}
 			}
@@ -1040,38 +1055,53 @@ void fsq::process_symbol(int sym)
 
 void fsq::process_tones()
 {
-	noise = 0;
 	max = 0;
-	peak = 0;
+	peak = NUMBINS / 2;
+
 // examine FFT bin contents over bandwidth +/- ~ 50 Hz
 // 8 * 12000 / 2048 = 46.875 Hz
-	int firstbin = basetone - 21;
-//	double K = 0.133333;
-//	if (progdefaults.fsq_fastavg) K *= 3.0;
-// time domain moving average filter for each tone bin
+	int firstbin = frequency * FSQ_SYMLEN / samplerate - NUMBINS / 2;
+
+	double sigval = 0;
+
+	double mins[3];
+	double min = 3.0e8;
+	double temp;
+
+	mins[0] = mins[1] = mins[2] = 1.0e8;
 	for (int i = 0; i < NUMBINS; ++i) {
 		val = norm(fft_data[i + firstbin]);
+// looking for maximum signal
 		tones[i] = binfilt[i]->run(val);
 		if (tones[i] > max) {
 			max = tones[i];
 			peak = i;
 		}
+// looking for minimum signal in a 3 bin sequence
+		mins[2] = tones[i];
+		temp = mins[0] + mins[1] + mins[2];
+		mins[0] = mins[1];
+		mins[1] = mins[2];
+		if (temp < min) min = temp;
 	}
 
-	noise += (tones[0] + tones[NUMBINS - 1]) / 2.0;
-	noise *= FFTSIZE / 2.0;
+	sigval = tones[peak-1] + tones[peak] + tones[peak+1];
+	if (min == 0) min = 1e-10;
 
-	if (noise < 1e-8) noise = 1e-8;
+	s2n = 10 * log10( snfilt->run(sigval/min)) - 36.0;
 
-	s2n = 10 * log10(snfilt->run(tones[peak]/noise)) + 3.0;
+//	if (s2n > 0) s2n *= 1.3;  // very empirical
+
+	if (s2n <= 0) metric = 2 * (25 + s2n);
+	if (s2n > 0) metric = 50 * ( 1 + s2n / 45);
+	metric = clamp(metric, 0, 100);
 
-	metric = 2 * (s2n + 20);
-	metric = CLAMP(metric, 0, 100.0);  // -20 to +30 db range
 	display_metric(metric);
 
 	if (metric < progStatus.sldrSquelchValue && ch_sqlch_open)
 		ch_sqlch_open = false;
 
+// requires consecutive hits
 	if (peak == prev_peak) {
 		peak_counter++;
 	} else {
@@ -1712,18 +1742,21 @@ void sounder(void *)
 	std::string xmtstr = FSQBOL;
 	xmtstr.append(active_modem->fsq_mycall()).append(":").append(FSQEOT);
 	int numsymbols = xmtstr.length();
-	int xmtsecs = (int)(1.0 * numsymbols * (symlen / 4096.0) / SR);
+
+	int xmtsecs = (int)(1.0 * numsymbols * (fsq::symlen / 4096.0) / SR);
+
 	if (fsq_tx_text->eot()) {
 		std::string stime = ztime();
 		stime.erase(4);
 		stime.insert(2,":");
 		std::string sndx = "\nSounded @ ";
 		sndx.append(stime);
-		fsq_string = std::string("SOUNDEX:").append(sndx);
-		write_fsq_que(fsq_string);
-		fsq_xmt(sndx);
+		display_fsq_rx_text(sndx, FTextBase::ALTR);
+
+		fsq_xmt(" ");
 	}
-	Fl::repeat_timeout(sounder_secs + xmtsecs, sounder);
+
+	Fl::repeat_timeout(sounder_secs - xmtsecs, sounder);
 }
 
 void fsq_start_sounder()
