@@ -29,6 +29,9 @@
 #define _FILTER_H
 
 #include "complex.h"
+#ifndef M_PI
+#define M_PI (3.1415926535897932385)
+#endif
 
 //=====================================================================
 // FIR filters
