@@ -55,7 +55,7 @@
 #define DEFAULT_HWFALL 124
 //#define DEFAULT_HNOM 500
 #define WMIN 800
-#define HMIN 460 //500
+#define HMIN 460
 //#define Wwfall		(DEFAULT_HNOM + 2 * BEZEL)
 //#define DEFAULT_WNOM (Wwfall + 2* DEFAULT_SW)
 
@@ -77,7 +77,7 @@ extern int Hmacros;
 //#define Hrcvtxt		(Htext) / 2
 //#define Hxmttxt		(Htext - (Hrcvtxt))
 
-#define Wmode 		80
+#define Wmode 		110
 #define Ws2n  		120
 #define Wimd  		120
 #define Wwarn 		16
