@@ -55,6 +55,7 @@ enum {
 
 	MODE_CONTESTIA,
 
+	MODE_DOMINOEXMICRO,
 	MODE_DOMINOEX4,
 	MODE_DOMINOEX5,
 	MODE_DOMINOEX8,
@@ -165,6 +166,7 @@ enum {
 
 	MODE_RTTY,
 
+	MODE_THORMICRO,
 	MODE_THOR4,
 	MODE_THOR5,
 	MODE_THOR8,
