@@ -71,7 +71,8 @@ extern	void	deleteTxViewer();
 
 extern void cb_picRxClose( Fl_Widget *w, void *);
 extern void cb_picRxAbort( Fl_Widget *w, void *);
-extern void picTxSendColor();
+extern void pic_TxSendColor();
+extern void pic_TxSendGrey();
 extern void cb_picTxSendColor( Fl_Widget *w, void *);
 extern void cb_picTxSendGrey( Fl_Widget *w, void *);
 extern void cb_picTxSendAbort( Fl_Widget *w, void *);
@@ -114,11 +115,12 @@ friend void updateTxPic(unsigned char data);
 friend void cb_picRxClose( Fl_Widget *w, void *);
 friend void cb_picRxAbort( Fl_Widget *w, void *);
 friend void pic_TxSendColor();
+friend void pic_TxSendGrey();
 friend void cb_picTxSendColor( Fl_Widget *w, void *);
 friend void cb_picTxSendGrey( Fl_Widget *w, void *);
 friend void cb_picTxSendAbort( Fl_Widget *w, void *);
 friend void cb_picTxSPP( Fl_Widget *w, void *);
-friend void load_image(const char *n);
+friend int  load_image(const char *n);
 
 public:
 enum {
@@ -271,7 +273,8 @@ public:
 	void	restart() {};
 	int		rx_process(const double *buf, int len);
 	int		tx_process();
-	void	send_image(std::string s);
+	void	send_color_image(std::string s);
+	void	send_Grey_image(std::string s);
 	void	shutdown();
 };
 
