@@ -83,6 +83,10 @@ private:
 	double			*fir2c;
 	Cmovavg			*snfilt;
 	Cmovavg			*imdfilt;
+	Cmovavg			*e0_filt;
+	Cmovavg			*e1_filt;
+	Cmovavg			*e2_filt;
+	bool			displaysn;
 
 	double			I1[NUM_FILTERS];
 	double			I2[NUM_FILTERS];
@@ -166,6 +170,9 @@ private:
 
 	double			*tx_shape;
 	int 			preamble;
+	C_FIR_filter	*xmtfilt;
+
+	void			transmit(double *buf, int len);
 	void			tx_carriers();
 	void 			tx_symbol(int sym);
 	void			tx_bit(int bit);
@@ -201,7 +208,11 @@ public:
 	void searchDown();
 	void searchUp();
 
-	void clear_viewer() { pskviewer->clear(); }
+	void clear_viewer() { 
+		if (pskviewer) pskviewer->clear();
+//		if ((mode >= MODE_PSK31 && mode <= MODE_PSK125) ||
+//			(mode >= MODE_QPSK31 && mode <= MODE_QPSK125)) pskviewer->clear(); 
+	}
 	void clear_ch(int n) { pskviewer->clearch(n); }
 	int  viewer_get_freq(int n) {
 		if (pskviewer) pskviewer->get_freq(n);
