@@ -123,6 +123,13 @@ public:
 		const char *mode, bool chkmode,
 		const char *xchg1, bool chkxchg1 );
 
+	int matched( cQsoRec *rec );
 };
 
+extern int comparebydate(const void *p1, const void *p2);
+extern int comparebymode(const void *p1, const void *p2);
+extern int comparebycall(const void *p1, const void *p2);
+extern int comparebyfreq(const void *p1, const void *p2);
+
+
 #endif
