@@ -56,7 +56,7 @@ extern void rigCAT_set_ptt(int);
 extern void rigCAT_set_qsy(long long f);
 
 extern void rigCAT_defaults();
-extern void rigCAT_restore_defaults();
+//extern void rigCAT_restore_defaults();
 
 // no xcvr 
 
