@@ -44,11 +44,13 @@ int pthread_cond_timedwait_rel(pthread_cond_t* cond, pthread_mutex_t* mutex, dou
 enum {
 	INVALID_TID = -1,
 	TRX_TID,
+	TOD_TID,
 	QRZ_TID,
 	RIGCTL_TID,
 	NORIGCTL_TID,
 	EQSL_TID,
 	ADIF_RW_TID,
+	ADIF_MERGE_TID,
 	XMLRPC_TID,
 	ARQ_TID,
 	ARQSOCKET_TID,
@@ -58,6 +60,8 @@ enum {
 	PSM_TID,
 	FD_TID,
 	N3FJP_TID,
+	DXCC_TID,
+	WKEY_TID,
 	FLMAIN_TID,
 	NUM_THREADS,
 	NUM_QRUNNER_THREADS = NUM_THREADS - 1
