@@ -133,6 +133,10 @@ extern "C" {
 const__ uint32_t ceil2(uint32_t n);
 const__ uint32_t floor2(uint32_t n);
 
+#if !HAVE_STRNLEN
+size_t strnlen(const char* str, size_t len);
+#endif
+
 #if !HAVE_STRCASESTR
 char* strcasestr(const char* haystack, const char* needle);
 #endif
