@@ -54,44 +54,58 @@ static const char *szEOR = "<EOR>";
 
 // These ADIF fields define the ADIF database
 FIELD fields[] = {
-//  TYPE,          FSIZE, NAME,           WIDGET
-	{FREQ,         12,    "FREQ",         &btnSelectFreq},      // QSO frequency in Mhz
-	{CALL,         15,    "CALL",         &btnSelectCall},      // contacted stations CALLSIGN
-	{MODE,         10,    "MODE",         &btnSelectMode},      // QSO mode
-	{NAME,         80,    "NAME",         &btnSelectName},      // contacted operators NAME
-	{QSO_DATE,     8,     "QSO_DATE",     &btnSelectQSOdateOn}, // QSO data
-	{QSO_DATE_OFF, 8,     "QSO_DATE_OFF", &btnSelectQSOdateOff},// QSO data OFF, according to ADIF 2.2.6
-	{TIME_OFF,     6,     "TIME_OFF",     &btnSelectTimeOFF},   // HHMM or HHMMSS in UTC
-	{TIME_ON,      6,     "TIME_ON",      &btnSelectTimeON},    // HHMM or HHMMSS in UTC
-	{QTH,          100,   "QTH",          &btnSelectQth},       // contacted stations city
-	{RST_RCVD,     3,     "RST_RCVD",     &btnSelectRSTrcvd},   // received signal report
-	{RST_SENT,     3,     "RST_SENT",     &btnSelectRSTsent},   // sent signal report
-	{STATE,        20,    "STATE",        &btnSelectState},     // contacted stations STATE
-	{VE_PROV,      20,    "VE_PROV",      &btnSelectProvince},  // 2 letter abbreviation for Canadian Province
-	{NOTES,        512,   "NOTES",        &btnSelectNotes},     // QSO notes
-	{QSLRDATE,     8,     "QSLRDATE",     &btnSelectQSLrcvd},   // QSL received date
-	{QSLSDATE,     8,     "QSLSDATE",     &btnSelectQSLsent},   // QSL sent date
-	{GRIDSQUARE,   8,     "GRIDSQUARE",   &btnSelectLOC},       // contacted stations Maidenhead Grid Square
-	{BAND,         8,     "BAND",         &btnSelectBand},      // QSO band
-	{CNTY,         60,    "CNTY",         &btnSelectCNTY},      // secondary political subdivision, ie: county
-	{COUNTRY,      60,    "COUNTRY",      &btnSelectCountry},   // contacted stations DXCC entity name
-	{CQZ,          8,     "CQZ",          &btnSelectCQZ},       // contacted stations CQ Zone
-	{DXCC,         8,     "DXCC",         &btnSelectDXCC},      // contacted stations Country Code
-	{QSL_VIA,      256,   "QSL_VIA",      &btnSelectQSL_VIA},   // contacted stations path
-	{IOTA,         20,    "IOTA",         &btnSelectIOTA},      // Islands on the air
-	{ITUZ,         20,    "ITUZ",         &btnSelectITUZ},      // ITU zone
-	{CONT,         60,    "CONT",         &btnSelectCONT},      // contacted stations continent
+//  TYPE,          FSIZE,  NAME,              WIDGET
+	{FREQ,         12,    "FREQ",             &btnSelectFreq},      // QSO frequency in Mhz
+	{CALL,         15,    "CALL",             &btnSelectCall},      // contacted stations CALLSIGN
+	{MODE,         10,    "MODE",             &btnSelectMode},      // QSO mode
+	{NAME,         80,    "NAME",             &btnSelectName},      // contacted operators NAME
+	{QSO_DATE,     8,     "QSO_DATE",         &btnSelectQSOdateOn}, // QSO data
+	{QSO_DATE_OFF, 8,     "QSO_DATE_OFF",     &btnSelectQSOdateOff},// QSO data OFF, according to ADIF 2.2.6
+	{TIME_OFF,     6,     "TIME_OFF",         &btnSelectTimeOFF},   // HHMM or HHMMSS in UTC
+	{TIME_ON,      6,     "TIME_ON",          &btnSelectTimeON},    // HHMM or HHMMSS in UTC
+	{QTH,          100,   "QTH",              &btnSelectQth},       // contacted stations city
+	{RST_RCVD,     3,     "RST_RCVD",         &btnSelectRSTrcvd},   // received signal report
+	{RST_SENT,     3,     "RST_SENT",         &btnSelectRSTsent},   // sent signal report
+	{STATE,        20,    "STATE",            &btnSelectState},     // contacted stations STATE
+	{VE_PROV,      20,    "VE_PROV",          &btnSelectProvince},  // 2 letter abbreviation for Canadian Province
+	{NOTES,        512,   "NOTES",            &btnSelectNotes},     // QSO notes
 
-	{SRX,          50,    "SRX",          &btnSelectSerialIN},  // received serial number for a contest QSO
-	{STX,          50,    "STX",          &btnSelectSerialOUT}, // QSO transmitted serial number
+	{QSLRDATE,     8,     "QSLRDATE",         &btnSelectQSLrcvd},   // QSL received date
+	{QSLSDATE,     8,     "QSLSDATE",         &btnSelectQSLsent},   // QSL sent date
 
-	{XCHG1,        100,   "SRX_STRING",   &btnSelectXchgIn},    // contest exchange #1 / free1 in xlog
-	{MYXCHG,       100,   "STX_STRING",   &btnSelectMyXchg},    // contest exchange sent
+	{EQSLRDATE,    8,     "EQSLRDATE",        NULL},                // EQSL received date
+	{EQSLSDATE,    8,     "EQSLSDATE",        NULL},                // EQSL sent date
 
-	{FDCLASS,      20,    "FD_CLASS",     &btnSelectClass},     // Field Day class received
-	{FDSECTION,    20,    "FD_SECTION",   &btnSelectSection},   // Field Day section received
+	{LOTWRDATE,    8,     "LOTWRDATE",        NULL},                // EQSL received date
+	{LOTWSDATE,    8,     "LOTWSDATE",        NULL},                // EQSL sent date
+
+	{GRIDSQUARE,   8,     "GRIDSQUARE",       &btnSelectLOC},       // contacted stations Maidenhead Grid Square
+	{BAND,         8,     "BAND",             &btnSelectBand},      // QSO band
+	{CNTY,         60,    "CNTY",             &btnSelectCNTY},      // secondary political subdivision, ie: county
+	{COUNTRY,      60,    "COUNTRY",          &btnSelectCountry},   // contacted stations DXCC entity name
+	{CQZ,          8,     "CQZ",              &btnSelectCQZ},       // contacted stations CQ Zone
+	{DXCC,         8,     "DXCC",             &btnSelectDXCC},      // contacted stations Country Code
+	{QSL_VIA,      256,   "QSL_VIA",          &btnSelectQSL_VIA},   // contacted stations path
+	{IOTA,         20,    "IOTA",             &btnSelectIOTA},      // Islands on the air
+	{ITUZ,         20,    "ITUZ",             &btnSelectITUZ},      // ITU zone
+	{CONT,         60,    "CONT",             &btnSelectCONT},      // contacted stations continent
+
+	{SRX,          50,    "SRX",              &btnSelectSerialIN},  // received serial number for a contest QSO
+	{STX,          50,    "STX",              &btnSelectSerialOUT}, // QSO transmitted serial number
+
+	{XCHG1,        100,   "SRX_STRING",       &btnSelectXchgIn},    // contest exchange #1 / free1 in xlog
+	{MYXCHG,       100,   "STX_STRING",       &btnSelectMyXchg},    // contest exchange sent
+
+	{FDCLASS,      20,    "FD_CLASS",         &btnSelectClass},     // Field Day class received
+	{FDSECTION,    20,    "FD_SECTION",       &btnSelectSection},   // Field Day section received
+
+	{TX_PWR,       8,     "TX_PWR",           &btnSelectTX_pwr},    // power transmitted by this station
+
+	{OP_CALL,     15,     "OPERATOR",         &btnSelectOperator},  // Callsign of person logging the QSO
+	{STA_CALL,    15,     "STATION_CALLSIGN", &btnSelectStaCall},   // Callsign of transmitting station
+	{MY_GRID,      8,     "MY_GRIDSQUARE",    &btnSelectStaGrid},   // Xmt station locator
+	{MY_CITY,     60,     "MY_CITY",          &btnSelectStaCity},   // Xmt station location
 
-	{TX_PWR,       8,     "TX_PWR",       &btnSelectTX_pwr},    // power transmitted by this station
 	{NUMFIELDS,    0,     "",             NULL}
 };
 
@@ -107,7 +121,6 @@ FIELD fields[] = {
 	{AGE,          2,     "AGE",          NULL},                // contacted operators age in years
 	{ARRL_SECT,    20,    "ARRL_SECT",    NULL},                // contacted stations ARRL section
 	{CONTEST_ID,   20,    "CONTEST_ID",   NULL},                // QSO contest identifier
-	{OPERATOR,     20,    "OPERATOR",     NULL},                // Callsign of person logging the QSO
 	{PFX,          20,    "PFX",          NULL},                // WPA prefix
 	{PROP_MODE,    100,   "PROP_MODE",    NULL},                // propogation mode
 	{QSL_MSG,      256,   "QSL_MSG",      NULL},                // personal message to appear on qsl card
@@ -303,7 +316,7 @@ void cAdifIO::do_readfile(const char *fname, cQsoDb *db)
 			} else {
 				ptr2 = ptr + 1;
 			}
-			if ((ptr2) > 0 && (unsigned)(ptr2 - ptr) <= p)
+			if ((ptr2 != NULL) && (unsigned)(ptr2 - ptr) <= p)
 				ptr = strchr(ptr2,'<');
 			else
 				break; // corrupt record
@@ -331,22 +344,22 @@ void cAdifIO::do_readfile(const char *fname, cQsoDb *db)
 	clock_gettime(CLOCK_REALTIME, &t1);
 #endif
 
-	t0 = t1 - t0;
-	float t = (t0.tv_sec + t0.tv_nsec/1e9);
+	float t = t1.tv_sec - t0.tv_sec + (t1.tv_nsec - t0.tv_nsec)/1e9;
+
 
 	if (!feof(adiFile))
 		snprintf(szmsg, sizeof(szmsg), "\
 ================================================\n\
 ERROR reading logbook %s\n\
-      read %d records in %4.2f seconds\n\
+      read %d records in %4.1f seconds\n\
 ================================================\n", fname, db->nbrRecs(), t);
 	else {
 		snprintf(szmsg, sizeof(szmsg), "\
 ================================================\n\
 Read Logbook: %s\n\
-      read %d records in %4.2f seconds\n\
+      read %d records in %4.1f seconds\n\
 ================================================\n", fname, db->nbrRecs(), t);
-		LOG_INFO("logfile: %s, read %d records in %4.2f seconds", fname, db->nbrRecs(), t);
+		LOG_INFO("logfile: %s, read %d records in %4.1f seconds", fname, db->nbrRecs(), t);
 		if (num_read_errors) {
 			if (!read_errors.empty()) {
 				read_errors.append("\n");
@@ -479,6 +492,47 @@ static cQsoDb *wrdb = 0;
 
 static struct timespec t0, t1;
 
+std::string cAdifIO::adif_record(cQsoRec *rec)
+{
+	static std::string record;
+	static std::string sFld;
+	record.clear();
+	for (int j = 0; fields[j].type != NUMFIELDS; j++) {
+		if (strcmp(fields[j].name,"MYXCHG") == 0) continue;
+		if (strcmp(fields[j].name,"XCHG1") == 0) continue;
+		sFld = rec->getField(fields[j].type);
+		if (!sFld.empty()) {
+			snprintf(recfield, sizeof(recfield),
+				adifmt,
+				fields[j].name,
+				sFld.length());
+			record.append(recfield).append(sFld);
+		}
+	}
+	record.append(szEOR);
+	record.append(szEOL);
+	return record;
+}
+
+int cAdifIO::writeAdifRec (cQsoRec *rec, const char *fname)
+{
+	std::string strRecord = adif_record(rec);
+
+	FILE *adiFile = fl_fopen (fname, "ab");
+
+	if (!adiFile) {
+		LOG_ERROR("Cannot write to %s", fname);
+		return 1;
+	}
+	LOG_INFO("Write record to %s", fname);
+
+	fprintf (adiFile, "%s", strRecord.c_str());
+
+	fclose (adiFile);
+
+	return 0;
+}
+
 int cAdifIO::writeLog (const char *fname, cQsoDb *db, bool immediate) {
 	ENSURE_THREAD(FLMAIN_TID);
 
@@ -521,14 +575,9 @@ void cAdifIO::do_writelog()
 	ADIFHEADER.append(szEOL);
 	ADIFHEADER.append("<PROGRAMVERSION:%d>%s");
 	ADIFHEADER.append(szEOL);
-	ADIFHEADER.append("<DATA CHECKSUM:%d>%s");
-	ADIFHEADER.append(szEOL);
 	ADIFHEADER.append("<EOH>");
 	ADIFHEADER.append(szEOL);
 
-	Ccrc16 checksum;
-	string s_checksum;
-
 	adiFile = fl_fopen (adif_file_name.c_str(), "wb");
 
 	if (!adiFile) {
@@ -538,41 +587,21 @@ void cAdifIO::do_writelog()
 	}
 	LOG_INFO("Writing %s", adif_file_name.c_str());
 
-	string sFld;
 	cQsoRec *rec;
 
 	records.clear();
 	for (int i = 0; i < adifdb->nbrRecs(); i++) {
 		rec = adifdb->getRec(i);
-		record.clear();
-		int j = 0;
-		while (fields[j].type != NUMFIELDS) {
-			if (strcmp(fields[j].name,"MYXCHG") == 0) { j++; continue; }
-			if (strcmp(fields[j].name,"XCHG1") == 0) { j++; continue; }
-			sFld = rec->getField(fields[j].type);
-			if (!sFld.empty()) {
-				snprintf(recfield, sizeof(recfield), adifmt,
-					fields[j].name,
-					sFld.length());
-				record.append(recfield).append(sFld);
-			}
-			j++;
-		}
-		record.append(szEOR);
-		record.append(szEOL);
-		records.append(record);
+		records.append(adif_record(rec));
 		adifdb->qsoUpdRec(i, rec);
 	}
 	nrecs = adifdb->nbrRecs();
 
-	s_checksum = checksum.scrc16(records);
-
 	fprintf (adiFile, ADIFHEADER.c_str(),
 		 fl_filename_name(adif_file_name.c_str()),
 		 strlen(ADIF_VERS), ADIF_VERS,
 		 strlen(PACKAGE_NAME), PACKAGE_NAME,
-		 strlen(PACKAGE_VERSION), PACKAGE_VERSION,
-		 s_checksum.length(), s_checksum.c_str()
+		 strlen(PACKAGE_VERSION), PACKAGE_VERSION
 		);
 	fprintf (adiFile, "%s", records.c_str());
 
