@@ -87,21 +87,83 @@ int n3fjp_has_xcvr_control = UNKNOWN;
 double tracked_freq = 0;
 int  tracked_mode = -1;
 
-enum {FJP_NONE, FJP_FD, FJP_CQWWRTTY};
+enum {FJP_NONE, FJP_FD, FJP_CQWWRTTY, FJP_NAQP, FJP_GENERIC};
 int  n3fjp_contest = FJP_NONE;
 
+int n3fjp_wait = 0;
+
+void adjust_freq(string s);
+void n3fjp_parse_response(string s);
+void n3fjp_disp_report(string s, string fm = "");
+void n3fjp_send(string cmd);
+void n3fjp_rcv(string &rx);
+void n3fjp_send_freq_mode();
+void n3fjp_clear_record();
+void n3fjp_getfields();
+void n3fjp_get_record(string call);
+static string ParseField(string &record, string fieldtag);
+static string ParseTextField(string &record, string fieldtag);
+static string ucasestr(string s);
+static void n3fjp_parse_data_stream(string buffer);
+static void n3fjp_parse_calltab_event(string buffer);
+string fmt_date(string date);
+string fmt_time(string time);
+string field_rec(string fld, string val);
+static string n3fjp_tstmode();
+static string n3fjp_opmode();
+static string n3fjp_opband();
+static string n3fjp_freq();
+static void send_control(const string ctl, string val);
+static void send_action(const string action);
+static void send_command(const string command, string val="");
+bool n3fjp_dupcheck();
+static void n3fjp_send_data();
+static void n3fjp_send_data_norig();
+void get_n3fjp_frequency();
+void do_n3fjp_add_record_entries();
+void n3fjp_set_freq(long f);
+void n3fjp_set_ptt(int on);
+void n3fjp_add_record(cQsoRec &record);
+void n3fjp_parse_response(string tempbuff);
+void n3fjp_rcv_data();
+static void connect_to_n3fjp_server();
+void n3fjp_start();
+void n3fjp_restart();
+void n3fjp_disconnect();
+void *n3fjp_loop(void *args);
+void n3fjp_init(void);
+void n3fjp_close(void);
+
+//======================================================================
+//
+//======================================================================
+static std::string strip(std::string s)
+{
+	while (s.length() && (s[0] <= ' ')) s.erase(0,1);
+	while (s.length() && (s[s.length()-1] <= ' ')) s.erase(s.length()-1, 1);
+	return s;
+}
+
 //======================================================================
 //
 //======================================================================
 
 void adjust_freq(string sfreq)
 {
-	long long freq = (long long)(1e6*atof(sfreq.c_str()));
+	long freq;
+	size_t pp = sfreq.find(".");
+	if (pp == string::npos) return;
+
+	while ((sfreq.length() - pp) < 7) sfreq.append("0");
+	sfreq.erase(pp,1);
+	freq = atol(sfreq.c_str());
 
 	if (freq == 0) return;
 
+	wf->rfcarrier(freq);
+	wf->movetocenter();
 	show_frequency(freq);
-	sendFreq(freq);
+
 	return;
 
 	if (progdefaults.N3FJP_sweet_spot) {
@@ -115,25 +177,32 @@ void adjust_freq(string sfreq)
 		else if (active_modem->get_mode() < MODE_SSB)
 			afreq = progdefaults.PSKsweetspot;
 		else {
+			wf->rfcarrier(freq);
+			wf->movetocenter();
 			show_frequency(freq);
-			sendFreq(freq);
 			return;
 		}
 		freq -= (wf->USB() ? afreq : -afreq);
+		wf->rfcarrier(freq);
+		wf->movetocenter();
 		show_frequency(freq);
-		wf->Carrier(afreq);
-		sendFreq(freq);
-	} else {
-		show_frequency(freq);
-		sendFreq(freq);
+		return;
 	}
+	wf->rfcarrier(freq);
+	wf->movetocenter();
+	show_frequency(freq);
 }
 
 //======================================================================
 //
 //======================================================================
+void n3fjp_show(string s)
+{
+	txt_N3FJP_data->insert(s.c_str());
+	txt_N3FJP_data->redraw();
+}
 
-static void n3fjp_disp_report(string s, string fm = "")
+void n3fjp_disp_report(string s, string fm)
 {
 	string report = fm.append(s);
 	size_t p = report.find("\r\n");
@@ -150,15 +219,16 @@ static void n3fjp_disp_report(string s, string fm = "")
 
 	std::string pathname = TempDir;
 	pathname.append("n3fjp_data_stream.txt");
-	FILE *n3fjplog = fopen(pathname.c_str(), "a");
+	FILE *n3fjplog = fl_fopen(pathname.c_str(), "a");
 	fprintf(n3fjplog, "%s", report.c_str());
 	fclose(n3fjplog);
 
 }
 
-static void n3fjp_send(string cmd)
+void n3fjp_send(string cmd)
 {
 	try {
+//LOG_INFO("%s", cmd.c_str());
 		cmd.append("\r\n");
 		n3fjp_socket->send(cmd);
 		n3fjp_disp_report(cmd, "SEND:");
@@ -167,7 +237,7 @@ static void n3fjp_send(string cmd)
 	}
 }
 
-static void n3fjp_rcv(string &rx)
+void n3fjp_rcv(string &rx)
 {
 	try {
 		n3fjp_socket->recv(rx);
@@ -178,7 +248,7 @@ static void n3fjp_rcv(string &rx)
 }
 
 
-static void send_freq_mode()
+void n3fjp_send_freq_mode()
 {
 	if (!active_modem) return;
 
@@ -212,7 +282,8 @@ void n3fjp_clear_record()
 	string cmd = "<CMD><ACTION><VALUE>CLEAR</VALUE></CMD>";
 	try {
 		n3fjp_send(cmd);
-		MilliSleep(5);
+		n3fjp_wait = 100;
+//		MilliSleep(5);
 	} catch (const SocketException& e) {
 		LOG_ERROR("Error: %d, %s", e.error(), e.what());
 	}
@@ -228,6 +299,7 @@ void n3fjp_getfields()
 	string cmd ="<CMD><ALLFIELDSWITHVALUES></CMD>";
 	try {
 		n3fjp_send(cmd);
+		n3fjp_wait = 100;
 	} catch (const SocketException& e) {
 		LOG_ERROR("Error: %d, %s", e.error(), e.what());
 		n3fjp_calltab = false;
@@ -257,13 +329,14 @@ void n3fjp_get_record(string call)
 		n3fjp_send(cmd2);
 		n3fjp_calltab = false;
 
+		n3fjp_wait = 100;
+
 	} catch (const SocketException& e) {
 		LOG_ERROR("Error: %d, %s", e.error(), e.what());
 		n3fjp_calltab = false;
 	}
 }
 
-
 //======================================================================
 // parse string containing value, e.g.
 // <FREQ>14.01310</FREQ>
@@ -300,6 +373,23 @@ static string ParseTextField(string &record, string fieldtag)
 	return record.substr(p2, p3 - p2);
 }
 
+//======================================================================
+// parse value contents
+// <VALUE>valuestring</VALUE>
+//======================================================================
+static string ParseValueField(string field, string &record)
+{
+	string start = "<";
+	start.append(field).append("><VALUE>");
+	string endvalue = "</VALUE>";
+	size_t p1 = record.find(start);
+	size_t p2 = record.find(endvalue, p1);
+	if ((p1 == string::npos) || (p2 == string::npos) ||
+		(p2 < p1) ) return "";
+	p1 += start.length();
+	return record.substr(p1, p2 - p1);
+}
+
 static string ucasestr(string s)
 {
 	for (size_t n = 0; n < s.length(); n++) s[n] = toupper(s[n]);
@@ -318,9 +408,11 @@ static void n3fjp_parse_data_stream(string buffer)
 	inpState->value(ParseTextField(buffer, "STATE").c_str());
 	inpCountry->value(ParseTextField(buffer, "COUNTRYWORKED").c_str());
 	inpLoc->value(ParseTextField(buffer, "GRID").c_str());
-
 	adjust_freq(ParseTextField(buffer, "FREQUENCY"));
 
+	inp_CQzone->value(ParseTextField(buffer, "CQZONE").c_str());
+	inp_CQstate->value(ParseTextField(buffer, "STATE").c_str());
+
 // comments field does not contain \n delimiters
 // substitute \n for each '-'
 	string comments = ParseTextField(buffer, "COMMENTS");
@@ -345,6 +437,8 @@ static void n3fjp_parse_calltab_event(string buffer)
 {
 	inpCall->value(ParseField(buffer, "CALL").c_str());
 	inpCountry->value(ParseField(buffer, "COUNTRY").c_str());
+//	LOG_INFO("%s", buffer.c_str());
+	n3fjp_getfields();
 }
 
 //======================================================================
@@ -433,6 +527,17 @@ static string n3fjp_freq()
 {
 	if (!active_modem) return "";
 	float freq = qsoFreqDisp->value();
+	if (progdefaults.N3FJP_modem_carrier) {
+		if (ModeIsLSB(mode_info[active_modem->get_mode()].adif_name)) {
+			freq -= active_modem->get_txfreq();
+			if (active_modem->get_mode() == MODE_RTTY)
+				freq -= progdefaults.rtty_shift / 2;
+		} else {
+			freq += active_modem->get_txfreq();
+			if (active_modem->get_mode() == MODE_RTTY)
+				freq += progdefaults.rtty_shift / 2;
+		}
+	}
 	freq /= 1e6;
 	char szfreq[20];
 	snprintf(szfreq, sizeof(szfreq), "%f", freq);
@@ -448,10 +553,10 @@ static void send_control(const string ctl, string val)
 	cmd.append("</VALUE></CMD>");
 	try {
 		n3fjp_send(cmd);
+		n3fjp_wait = 100;
 	} catch (...) {
 		throw;
 	}
-	MilliSleep(10);
 }
 
 static void send_action(const string action)
@@ -462,13 +567,13 @@ static void send_action(const string action)
 	cmd.append("</VALUE></CMD>");
 	try {
 		n3fjp_send(cmd);
+		n3fjp_wait = 100;
 	} catch (...) {
 		throw;
 	}
-	MilliSleep(10);
 }
 
-static void send_command(const string command, string val="")
+static void send_command(const string command, string val)
 {
 	string cmd;
 	cmd.assign("<CMD><").append(command).append(">");
@@ -478,47 +583,56 @@ static void send_command(const string command, string val="")
 	try {
 		n3fjp_send(cmd);
 		MilliSleep(5);
+		n3fjp_wait = 100;
 	} catch (...) {
 		throw;
 	}
-	MilliSleep(10);
 }
 
+string last_lookup = "";
+bool   last_dupcheck = false;
+
 bool n3fjp_dupcheck()
 {
+	guard_lock rx_lock(&n3fjp_mutex);
+
 	string chkcall = inpCall->value();
+	if (chkcall == last_lookup) return last_dupcheck;
+
 	if (chkcall.length() < 3) return false;
+	if ((chkcall.length() == 3) && isdigit(chkcall[2])) return false;
+
+	last_lookup = chkcall;
 
 	string cmd;
+	bool isdup = false;
 
 	cmd.assign("<CMD><DUPECHECK><CALL>");
-	cmd.append(inpCall->value());
+	cmd.append(strip(inpCall->value()));
 	cmd.append("</CALL><BAND>");
-	cmd.append(n3fjp_opband());
+	cmd.append(strip(n3fjp_opband()));
 	cmd.append("</BAND><MODE>");
-	cmd.append(n3fjp_tstmode());
+	cmd.append(strip(n3fjp_tstmode()));
 	cmd.append("</MODE></CMD>");
 
-	guard_lock rx_lock(&n3fjp_mutex);
-
 	try {
 		n3fjp_send(cmd);
-	} catch (...) {
-		;
-	}
 
-	MilliSleep(100);
+		MilliSleep(200);
 
-	try {
 		string resp;
 		n3fjp_rcv(resp);
+
 		if (resp.find("Duplicate") != string::npos)
-			return true;
-		return false;
+			isdup = true;
+		n3fjp_parse_response(resp);
+
 	} catch (...) {
-		;
+		throw;
 	}
-	return false;
+	last_dupcheck = isdup;
+
+	return isdup;
 }
 
 static cQsoRec rec;
@@ -530,8 +644,7 @@ static void n3fjp_send_data()
 		send_control("FREQUENCY", n3fjp_freq());
 		send_control("BAND", n3fjp_opband());
 		send_control("MODE", n3fjp_opmode());
-		send_control("CALL", rec.getField(CALL));
-		send_action("CALLTAB");
+		send_control("CALL", strip(rec.getField(CALL)));
 
 		if (n3fjp_contest == FJP_NONE) {
 			send_control("DATE", fmt_date(rec.getField(QSO_DATE)));
@@ -548,23 +661,35 @@ static void n3fjp_send_data()
 		}
 		if (n3fjp_contest == FJP_FD) {
 			send_control("MODETST", n3fjp_tstmode());
-			send_control("CLASS", ucasestr(rec.getField(FDCLASS)));
-			send_control("SECTION", ucasestr(rec.getField(FDSECTION)));
-			send_control("SERIALR", rec.getField(SRX));
-			send_control("SERIALS", rec.getField(STX));
+			send_control("CLASS", strip(ucasestr(rec.getField(FDCLASS))));
+			send_control("SECTION", strip(ucasestr(rec.getField(FDSECTION))));
 		}
 		if (n3fjp_contest == FJP_CQWWRTTY) {
-			send_control("RSTS", rec.getField(RST_SENT));
-			send_control("RSTR", rec.getField(RST_RCVD));
-			send_control("CQZONE", rec.getField(CQZ));
-			send_control("STATE", rec.getField(STATE));
+			send_control("RSTS", strip(rec.getField(RST_SENT)));
+			send_control("RSTR", strip(rec.getField(RST_RCVD)));
+			send_control("CQZONE", strip(rec.getField(CQZ)));
+			send_control("STATE", strip(rec.getField(STATE)));
 		}
+		if (n3fjp_contest == FJP_GENERIC) {
+			send_control("RSTS", strip(rec.getField(RST_SENT)));
+			send_control("RSTR", strip(rec.getField(RST_RCVD)));
+			send_control("SERIALNOR", strip(rec.getField(SRX)));
+			send_control("SPCNUM", strip(ucasestr(rec.getField(XCHG1))));
+		}
+		if (n3fjp_contest == FJP_NAQP) {
+			send_control("NAMER", rec.getField(NAME));
+			if (strlen(rec.getField(STATE)) > 0)
+				send_control("SPCNUM", rec.getField(STATE));
+			else if (strlen(rec.getField(VE_PROV)) > 0)
+				send_control("SPCNUM", rec.getField(VE_PROV));
+		}
+
 		string other = "XCVR:";
 		char szfreq[6];
 		snprintf(szfreq, sizeof(szfreq), "%d", (int)active_modem->get_txfreq());
 		other.append(ModeIsLSB(rec.getField(MODE)) ? "LSB" : "USB");
 		other.append(" MODE:");
-		other.append(rec.getField(MODE));
+		other.append(strip(rec.getField(MODE)));
 		other.append(" WF:");
 		other.append(szfreq);
 
@@ -585,8 +710,7 @@ static void n3fjp_send_data_norig()
 		cmd.append("</CMD>");
 		n3fjp_send(cmd);
 
-		send_control("CALL", rec.getField(CALL));
-		send_action("CALLTAB");
+		send_control("CALL", strip(rec.getField(CALL)));
 
 		if (n3fjp_contest == FJP_NONE) {
 			send_control("FREQUENCY", n3fjp_freq());
@@ -594,28 +718,39 @@ static void n3fjp_send_data_norig()
 			send_control("DATE", fmt_date(rec.getField(QSO_DATE)));
 			send_control("TIMEON", fmt_time(rec.getField(TIME_ON)));
 			send_control("TIMEOFF", fmt_time(rec.getField(TIME_OFF)));
-			send_control("RSTS", rec.getField(RST_SENT));
-			send_control("RSTR", rec.getField(RST_RCVD));
-			send_control("NAMER", rec.getField(NAME));
-			send_control("COMMENTS", rec.getField(NOTES));
-			send_control("POWER", rec.getField(TX_PWR));
-			send_control("STATE", rec.getField(STATE));
-			send_control("GRIDR", rec.getField(GRIDSQUARE));
-			send_control("QTHGROUP", rec.getField(QTH));
+			send_control("RSTS", strip(rec.getField(RST_SENT)));
+			send_control("RSTR", strip(rec.getField(RST_RCVD)));
+			send_control("NAMER", strip(rec.getField(NAME)));
+			send_control("COMMENTS", strip(rec.getField(NOTES)));
+			send_control("POWER", strip(rec.getField(TX_PWR)));
+			send_control("STATE", strip(rec.getField(STATE)));
+			send_control("GRIDR", strip(rec.getField(GRIDSQUARE)));
+			send_control("QTHGROUP", strip(rec.getField(QTH)));
 		}
 		if (n3fjp_contest == FJP_FD) {
 			send_control("MODETST", n3fjp_tstmode());
-			send_control("CLASS", ucasestr(rec.getField(FDCLASS)));
-			send_control("SECTION", ucasestr(rec.getField(FDSECTION)));
-			send_control("SERIALR", rec.getField(SRX));
-			send_control("SERIALS", rec.getField(STX));
+			send_control("CLASS", strip(ucasestr(rec.getField(FDCLASS))));
+			send_control("SECTION", strip(ucasestr(rec.getField(FDSECTION))));
 		}
 		if (n3fjp_contest == FJP_CQWWRTTY) {
 			send_control("MODETST", n3fjp_tstmode());
-			send_control("RSTS", rec.getField(RST_SENT));
-			send_control("RSTR", rec.getField(RST_RCVD));
-			send_control("CQZONE", rec.getField(CQZ));
-			send_control("STATE", rec.getField(STATE));
+			send_control("RSTS", strip(rec.getField(RST_SENT)));
+			send_control("RSTR", strip(rec.getField(RST_RCVD)));
+			send_control("CQZONE", strip(rec.getField(CQZ)));
+			send_control("STATE", strip(rec.getField(STATE)));
+		}
+		if (n3fjp_contest == FJP_GENERIC) {
+			send_control("RSTS", strip(rec.getField(RST_SENT)));
+			send_control("RSTR", strip(rec.getField(RST_RCVD)));
+			send_control("SERIALNOR", strip(rec.getField(SRX)));
+			send_control("SPCNUM", strip(ucasestr(rec.getField(XCHG1))));
+		}
+		if (n3fjp_contest == FJP_NAQP) {
+			send_control("NAMER", rec.getField(NAME));
+			if (strlen(rec.getField(STATE)) > 0)
+				send_control("SPCNUM", rec.getField(STATE));
+			else if (strlen(rec.getField(VE_PROV)) > 0)
+				send_control("SPCNUM", rec.getField(VE_PROV));
 		}
 
 		string other = "XCVR:";
@@ -623,7 +758,7 @@ static void n3fjp_send_data_norig()
 		snprintf(szfreq, sizeof(szfreq), "%d", (int)active_modem->get_txfreq());
 		other.append(ModeIsLSB(rec.getField(MODE)) ? "LSB" : "USB");
 		other.append(" MODE:");
-		other.append(rec.getField(MODE));
+		other.append(strip(rec.getField(MODE)));
 		other.append(" WF:");
 		other.append(szfreq);
 
@@ -667,11 +802,15 @@ void do_n3fjp_add_record_entries()
 void n3fjp_set_freq(long f)
 {
 	char szfreq[20];
-	snprintf(szfreq, sizeof(szfreq), "%f", f/1e6);
+	snprintf(szfreq, sizeof(szfreq), "%ld", f);
+	string freq = szfreq;
+	while (freq.length() < 7) freq.insert(0, "0");
+	freq.insert(freq.length() - 6, ".");
+
 	string cmd;
 
 	cmd.assign("<CMD><CHANGEFREQ><VALUE>");
-	cmd.append(szfreq);
+	cmd.append(freq);
 	cmd.append("</VALUE><SUPPRESSMODEDEFAULT>TRUE</SUPPRESSMODEDEFAULT></CMD>");
 
 	{	guard_lock send_lock(&send_this_mutex);
@@ -697,10 +836,78 @@ void n3fjp_set_ptt(int on)
 
 void n3fjp_add_record(cQsoRec &record)
 {
+	if (!n3fjp_connected) return;
 	rec = record;
 	n3fjp_bool_add_record = true;
 }
 
+void n3fjp_request_next_serial_number()
+{
+	send_command("<CMD><NEXTSERIALNUMBER></CMD>");
+}
+
+string n3fjp_serno = "";
+
+void n3fjp_parse_next_serial(string buff)
+{
+	n3fjp_serno = ParseValueField("NEXTSERIALNUMBERRESPONSE", buff);
+	clearQSO();
+}
+
+//======================================================================
+//
+//======================================================================
+void n3fjp_parse_response(string tempbuff)
+{
+	if (tempbuff.empty()) return;
+	size_t p1 = string::npos, p2 = string::npos;
+
+	if (tempbuff.find("RIGRESPONSE") != string::npos) {
+		size_t p0 = tempbuff.find("<RIG>");
+		if (p0 != string::npos) {
+			p0 += strlen("<RIG>");
+			string rigname = tempbuff.substr(p0);
+			p0 = rigname.find("</RIG>");
+			if (p0 != string::npos) {
+				rigname.erase(p0);
+				if (rigname != "None" && rigname != "Client API") {
+					n3fjp_has_xcvr_control = N3FJP;
+					send_command("READBMF");
+				} else
+				n3fjp_has_xcvr_control = FLDIGI;
+			}
+		}
+	}
+
+	if (n3fjp_has_xcvr_control == N3FJP) {
+		if ((p1 = tempbuff.find("<CHANGEFREQ><VALUE>")) != string::npos) {
+			p1 += strlen("<CHANGEFREQ><VALUE>");
+			p2 = tempbuff.find("</VALUE>", p1);
+			if (p2 == string::npos) return;
+			string sfreq = tempbuff.substr(p1, p2 - p1);
+			REQ(adjust_freq, sfreq);
+		} else if (tempbuff.find("<READBMFRESPONSE>") != string::npos) {
+			string sfreq = ParseField(tempbuff, "FREQ");
+			REQ(adjust_freq, sfreq);
+		}
+	}
+
+	if (tempbuff.find("<CALLTABEVENT>") != string::npos) {
+		n3fjp_rxbuffer = tempbuff;
+		REQ(n3fjp_parse_calltab_event, tempbuff);
+	}
+
+	if (tempbuff.find("ALLFIELDSWVRESPONSE") != string::npos) {
+		REQ(n3fjp_parse_data_stream, tempbuff);
+	}
+	if (tempbuff.find("<ENTEREVENT>") != string::npos) {
+		REQ(n3fjp_request_next_serial_number);
+	}
+	if (tempbuff.find("<NEXTSERIALNUMBERRESPONSE>") != string::npos) {
+		REQ(n3fjp_parse_next_serial, tempbuff);
+	}
+}
+
 //======================================================================
 //
 //======================================================================
@@ -708,48 +915,8 @@ void n3fjp_rcv_data()
 {
 	string tempbuff = "";
 	try {
-		size_t p1 = string::npos, p2 = string::npos;
 		n3fjp_rcv(tempbuff);
-		if (!tempbuff.empty()) {
-
-			if (tempbuff.find("RIGRESPONSE") != string::npos) {
-				size_t p0 = tempbuff.find("<RIG>");
-				if (p0 != string::npos) {
-					p0 += strlen("<RIG>");
-					string rigname = tempbuff.substr(p0);
-					p0 = rigname.find("</RIG>");
-					if (p0 != string::npos) {
-						rigname.erase(p0);
-						if (rigname != "None" && rigname != "Client API") {
-							n3fjp_has_xcvr_control = N3FJP;
-							send_command("READBMF");
-						} else
-						n3fjp_has_xcvr_control = FLDIGI;
-					}
-				}
-			}
-
-			if (n3fjp_has_xcvr_control == N3FJP) {
-				if ((p1 = tempbuff.find("<CHANGEFREQ><VALUE>")) != string::npos) {
-					p1 += strlen("<CHANGEFREQ><VALUE>");
-					p2 = tempbuff.find("</VALUE>", p1);
-					if (p2 == string::npos) return;
-					string sfreq = tempbuff.substr(p1, p2 - p1);
-					REQ(adjust_freq, sfreq);
-				} else if (tempbuff.find("<READBMFRESPONSE>") != string::npos) {
-					string sfreq = ParseField(tempbuff, "FREQ");
-					REQ(adjust_freq, sfreq);
-				}
-			}
-			if (tempbuff.find("<CALLTABEVENT>") != string::npos) {
-				n3fjp_rxbuffer = tempbuff;
-				REQ(n3fjp_parse_calltab_event, tempbuff);
-				REQ(n3fjp_getfields);
-			}
-			if (tempbuff.find("ALLFIELDSWVRESPONSE") != string::npos) {
-				REQ(n3fjp_parse_data_stream, tempbuff);
-			}
-		}
+		n3fjp_parse_response(tempbuff);
 	} catch (const SocketException& e) {
 		LOG_ERROR("Error %d, %s", e.error(), e.what());
 	}
@@ -758,18 +925,23 @@ void n3fjp_rcv_data()
 static void connect_to_n3fjp_server()
 {
 	try {
+		n3fjp_serno.clear();
+
 		if (!n3fjp_connected)
 			n3fjp_socket->connect();
 
+		if (!n3fjp_socket->is_connected()) return;
+
 		std::string pathname = TempDir;
 		pathname.append("n3fjp_data_stream.txt");
-		FILE *n3fjplog = fopen(pathname.c_str(), "w");
+		FILE *n3fjplog = fl_fopen(pathname.c_str(), "w");
 		fprintf(n3fjplog, "N3FJP / fldigi tcpip log\n\n");
 		fclose(n3fjplog);
 
 		string buffer;
 		string cmd = "<CMD><PROGRAM></CMD>";
 		n3fjp_send(cmd);
+		MilliSleep(100);
 
 		n3fjp_rcv(buffer);
 		if (buffer.empty()) return;
@@ -781,8 +953,14 @@ static void connect_to_n3fjp_server()
 			n3fjp_contest = FJP_NONE;
 		else if (info.find("CQ WW DX RTTY Contest Log") != string::npos)
 			n3fjp_contest = FJP_CQWWRTTY;
-		else if (info.find("ARRL Field Day Contest Log") != string::npos)
+		else if (info.find("Field Day Contest") != string::npos)
 			n3fjp_contest = FJP_FD;
+		else if (info.find("NAQP Contest Log") != string::npos)
+			n3fjp_contest = FJP_NAQP;
+		else {
+			n3fjp_contest = FJP_GENERIC;
+			REQ(n3fjp_request_next_serial_number);
+		}
 
 		info.insert(0, "Connected to ");
 
@@ -810,6 +988,10 @@ static void connect_to_n3fjp_server()
 		send_command(cmd);
 
 	} catch (const SocketException& e) {
+		string err;
+		err = e.what();
+		err.append("\n");
+		REQ(n3fjp_show, err);
 		LOG_INFO("%s(%d)", e.what(), e.error());
 	}
 
@@ -821,6 +1003,8 @@ static void connect_to_n3fjp_server()
 
 void n3fjp_start()
 {
+	static bool firstreport = true;
+
 	n3fjp_ip_address =  progdefaults.N3FJP_address;
 	n3fjp_ip_port = progdefaults.N3FJP_port;
 
@@ -833,9 +1017,13 @@ void n3fjp_start()
 		n3fjp_socket->set_timeout(0.01);
 		n3fjp_socket->set_nonblocking(true);
 		LOG_INFO("Client socket %d", n3fjp_socket->fd());
+		firstreport = true;
 	}
 	catch (const SocketException& e) {
-		LOG_INFO("%s", e.what() );
+		if (firstreport) {
+			LOG_INFO("%s", e.what() );
+			firstreport = false;
+		}
 		delete n3fjp_socket;
 		n3fjp_socket = 0;
 		n3fjp_connected = false;
@@ -845,6 +1033,8 @@ void n3fjp_start()
 
 void n3fjp_restart()
 {
+	static bool firstreport = true;
+
 	n3fjp_ip_address =  progdefaults.N3FJP_address;
 	n3fjp_ip_port = progdefaults.N3FJP_port;
 
@@ -860,9 +1050,13 @@ void n3fjp_restart()
 		n3fjp_socket->set_timeout(0.01);
 		n3fjp_socket->set_nonblocking(true);
 		LOG_INFO("Client socket %d", n3fjp_socket->fd());
+		firstreport = true;
 	}
 	catch (const SocketException& e) {
-		LOG_INFO("%s", e.what() );
+		if (firstreport) {
+			LOG_INFO("%s", e.what() );
+			firstreport = false;
+		}
 		delete n3fjp_socket;
 		n3fjp_socket = 0;
 		n3fjp_connected = false;
@@ -883,6 +1077,8 @@ void n3fjp_disconnect()
 	n3fjp_connected = false;
 	n3fjp_has_xcvr_control = UNKNOWN;
 	LOG_INFO("Disconnected");
+	n3fjp_serno.clear();
+	REQ(clearQSO);
 }
 
 //======================================================================
@@ -893,15 +1089,20 @@ void *n3fjp_loop(void *args)
 {
 	SET_THREAD_ID(N3FJP_TID);
 
-	int loopcount = 10;
 	int n3fjp_looptime = 100;
 	while(1) {
 		if (n3fjp_exit) break;
 
-		for (int i = 0; i < n3fjp_looptime/10; i++) {
-			MilliSleep(10);
-			if (n3fjp_exit) break;
-		}
+		MilliSleep(10);
+		if (n3fjp_wait) n3fjp_wait -= 10;
+		if (n3fjp_looptime) n3fjp_looptime -= 10;
+
+		if (n3fjp_wait > 0) continue;
+
+		if (n3fjp_looptime > 0) continue;
+
+		if (!n3fjp_connected) n3fjp_looptime = 5000;  // test for N3FJP logger every 5 sec
+		else n3fjp_looptime = 250;  // r/w to N3FJP logger every 1/4 second
 
 		if (!n3fjp_socket || (n3fjp_socket->fd() == -1)) {
 			n3fjp_start();
@@ -913,9 +1114,8 @@ void *n3fjp_loop(void *args)
 				n3fjp_restart();
 			}
 
-			else if (!n3fjp_connected && progdefaults.connect_to_n3fjp && (--loopcount == 0)) {
+			else if (!n3fjp_connected && progdefaults.connect_to_n3fjp ) {
 				connect_to_n3fjp_server();
-				loopcount = 1;
 			} 
 
 			else if (n3fjp_connected && !progdefaults.connect_to_n3fjp)
@@ -924,7 +1124,7 @@ void *n3fjp_loop(void *args)
 			else if (n3fjp_connected) {
 				try {
 					if (n3fjp_has_xcvr_control == FLDIGI)
-						send_freq_mode();
+						n3fjp_send_freq_mode();
 					if (!send_this.empty()) {
 						guard_lock send_lock(&send_this_mutex);
 						n3fjp_send(send_this);
@@ -953,6 +1153,7 @@ void *n3fjp_loop(void *args)
 //======================================================================
 //
 //======================================================================
+
 void n3fjp_init(void)
 {
 	n3fjp_enabled = false;
