@@ -99,6 +99,8 @@
 #include "psm/psm.h"
 #include "fd_logger.h"
 #include "n3fjp_logger.h"
+#include "dx_cluster.h"
+#include "dx_dialog.h"
 
 #if USE_HAMLIB
 	#include "rigclass.h"
@@ -139,6 +141,7 @@ string MacrosDir = "";
 string WrapDir = "";
 string TalkDir = "";
 string TempDir = "";
+string LoTWDir = "";
 string KmlDir = "";
 string PskMailDir = "";
 
@@ -210,21 +213,324 @@ static void fatal_error(string);
 #  define SHOW_WIZARD_BEFORE_MAIN_WINDOW 0
 #endif
 
+/*
+from: https://msdn.microsoft.com/en-us/library/windows/desktop/ms682425(v=vs.85).aspx
+
+BOOL WINAPI CreateProcess(
+  _In_opt_    LPCTSTR               lpApplicationName,
+  _Inout_opt_ LPTSTR                lpCommandLine,
+  _In_opt_    LPSECURITY_ATTRIBUTES lpProcessAttributes,
+  _In_opt_    LPSECURITY_ATTRIBUTES lpThreadAttributes,
+  _In_        BOOL                  bInheritHandles,
+  _In_        DWORD                 dwCreationFlags,
+  _In_opt_    LPVOID                lpEnvironment,
+  _In_opt_    LPCTSTR               lpCurrentDirectory,
+  _In_        LPSTARTUPINFO         lpStartupInfo,
+  _Out_       LPPROCESS_INFORMATION lpProcessInformation
+);
+
+Parameters
+
+lpApplicationName [in, optional]
+
+  The name of the module to be executed. This module can be a Windows-based
+  application. It can be some other type of module (for example, MS-DOS or OS/2)
+  if the appropriate subsystem is available on the local computer.
+
+  The string can specify the full path and file name of the module to execute
+  or it can specify a partial name. In the case of a partial name, the function
+  uses the current drive and current directory to complete the specification.
+  The function will not use the search path. This parameter must include the
+  file name extension; no default extension is assumed.
+
+  The lpApplicationName parameter can be NULL. In that case, the module name must
+  be the first white space–delimited token in the lpCommandLine string. If you are
+  using a long file name that contains a space, use quoted strings to indicate where
+  the file name ends and the arguments begin; otherwise, the file name is ambiguous.
+  For example, consider the string "c:\program files\sub dir\program name".
+  This string can be interpreted in a number of ways. The system tries to interpret
+  the possibilities in the following order:
+
+     c:\program.exe files\sub dir\program name
+     c:\program files\sub.exe dir\program name
+     c:\program files\sub dir\program.exe name
+     c:\program files\sub dir\program name.exe
+
+  If the executable module is a 16-bit application, lpApplicationName should be NULL,
+  and the string pointed to by lpCommandLine should specify the executable module as
+  well as its arguments.
+
+  To run a batch file, you must start the command interpreter; set lpApplicationName
+  to cmd.exe and set lpCommandLine to the following arguments: /c plus the name of the
+  batch file.
+
+lpCommandLine [in, out, optional]
+
+  The command line to be executed. The maximum length of this string is 32,768
+  characters, including the Unicode terminating null character. If lpApplicationName
+  is NULL, the module name portion of lpCommandLine is limited to MAX_PATH characters.
+
+  The Unicode version of this function, CreateProcessW, can modify the contents of this
+  string. Therefore, this parameter cannot be a pointer to read-only memory (such as a
+  const variable or a literal string). If this parameter is a constant string, the
+  function may cause an access violation.
+
+  The lpCommandLine parameter can be NULL. In that case, the function uses the string
+  pointed to by lpApplicationName as the command line.
+
+  If both lpApplicationName and lpCommandLine are non-NULL, the null-terminated string
+  pointed to by lpApplicationName specifies the module to execute, and the n
+  ull-terminated string pointed to by lpCommandLine specifies the command line. The
+  new process can use GetCommandLine to retrieve the entire command line. Console
+  processes written in C can use the argc and argv arguments to parse the command line.
+  Because argv[0] is the module name, C programmers generally repeat the module name
+  as the first token in the command line.
+
+  If lpApplicationName is NULL, the first white space–delimited token of the command
+  line specifies the module name. If you are using a long file name that contains a
+  space, use quoted strings to indicate where the file name ends and the arguments
+  begin (see the explanation for the lpApplicationName parameter). If the file name
+  does not contain an extension, .exe is appended. Therefore, if the file name
+  extension is .com, this parameter must include the .com extension. If the file
+  name ends in a period (.) with no extension, or if the file name contains a path,
+  .exe is not appended. If the file name does not contain a directory path, the
+  system searches for the executable file in the following sequence:
+
+    The directory from which the application loaded.
+    The current directory for the parent process.
+    The 32-bit Windows system directory. Use the GetSystemDirectory function to get
+    the path of this directory.
+    The 16-bit Windows system directory. There is no function that obtains the path
+    of this directory, but it is searched. The name of this directory is System.
+    The Windows directory. Use the GetWindowsDirectory function to get the path of
+    this directory.
+    The directories that are listed in the PATH environment variable. Note that this
+    function does not search the per-application path specified by the App Paths registry
+    key. To include this per-application path in the search sequence, use the ShellExecute
+    function.
+
+  The system adds a terminating null character to the command-line string to separate
+  the file name from the arguments. This divides the original string into two strings
+  for internal processing.
+lpProcessAttributes [in, optional]
+
+  A pointer to a SECURITY_ATTRIBUTES structure that determines whether the returned
+  handle to the new process object can be inherited by child processes. If
+  lpProcessAttributes is NULL, the handle cannot be inherited.
+
+  The lpSecurityDescriptor member of the structure specifies a security descriptor for
+  the new process. If lpProcessAttributes is NULL or lpSecurityDescriptor is NULL, the
+  process gets a default security descriptor. The ACLs in the default security descriptor
+  for a process come from the primary token of the creator.
+
+  Windows XP:  The ACLs in the default security descriptor for a process come from the
+  primary or impersonation token of the creator. This behavior changed with Windows XP
+  with SP2 and Windows Server 2003.
+
+lpThreadAttributes [in, optional]
+
+  A pointer to a SECURITY_ATTRIBUTES structure that determines whether the returned
+  handle to the new thread object can be inherited by child processes. If
+  lpThreadAttributes is NULL, the handle cannot be inherited.
+
+  The lpSecurityDescriptor member of the structure specifies a security descriptor for
+  the main thread. If lpThreadAttributes is NULL or lpSecurityDescriptor is NULL, the
+  thread gets a default security descriptor. The ACLs in the default security descriptor
+  for a thread come from the process token.
+
+  Windows XP:  The ACLs in the default security descriptor for a thread come from the
+  primary or impersonation token of the creator. This behavior changed with Windows XP
+  with SP2 and Windows Server 2003.
+
+bInheritHandles [in]
+
+  If this parameter TRUE, each inheritable handle in the calling process is inherited by
+  the new process. If the parameter is FALSE, the handles are not inherited. Note that
+  inherited handles have the same value and access rights as the original handles.
+
+dwCreationFlags [in]
+
+  The flags that control the priority class and the creation of the process. For a list
+  of values, see Process Creation Flags.
+
+  This parameter also controls the new process's priority class, which is used to
+  determine the scheduling priorities of the process's threads. For a list of values,
+  see GetPriorityClass. If none of the priority class flags is specified, the priority
+  class defaults to NORMAL_PRIORITY_CLASS unless the priority class of the creating
+  process is IDLE_PRIORITY_CLASS or BELOW_NORMAL_PRIORITY_CLASS. In this case, the
+  child process receives the default priority class of the calling process.
+
+lpEnvironment [in, optional]
+
+  A pointer to the environment block for the new process. If this parameter is NULL,
+  the new process uses the environment of the calling process.
+
+  An environment block consists of a null-terminated block of null-terminated strings.
+  Each string is in the following form:
+
+  name=value\0
+
+  Because the equal sign is used as a separator, it must not be used in the name of
+  an environment variable.
+
+  An environment block can contain either Unicode or ANSI characters. If the
+  environment block pointed to by lpEnvironment contains Unicode characters, be
+  sure that dwCreationFlags includes CREATE_UNICODE_ENVIRONMENT. If this parameter
+  is NULL and the environment block of the parent process contains Unicode characters,
+  you must also ensure that dwCreationFlags includes CREATE_UNICODE_ENVIRONMENT.
+
+  The ANSI version of this function, CreateProcessA fails if the total size of the
+  environment block for the process exceeds 32,767 characters.
+
+  Note that an ANSI environment block is terminated by two zero bytes: one for the
+  last string, one more to terminate the block. A Unicode environment block is
+  terminated by four zero bytes: two for the last string, two more to terminate
+  the block.
+
+lpCurrentDirectory [in, optional]
+
+  The full path to the current directory for the process. The string can also
+  specify a UNC path.
+
+  If this parameter is NULL, the new process will have the same current drive and
+  directory as the calling process. (This feature is provided primarily for shells
+  that need to start an application and specify its initial drive and working
+  directory.)
+
+lpStartupInfo [in]
+
+  A pointer to a STARTUPINFO or STARTUPINFOEX structure.
+
+  To set extended attributes, use a STARTUPINFOEX structure and specify
+  EXTENDED_STARTUPINFO_PRESENT in the dwCreationFlags parameter.
+
+  Handles in STARTUPINFO or STARTUPINFOEX must be closed with CloseHandle when
+  they are no longer needed.
+  Important  The caller is responsible for ensuring that the standard handle
+  fields in STARTUPINFO contain valid handle values. These fields are copied
+  unchanged to the child process without validation, even when the dwFlags
+  member specifies STARTF_USESTDHANDLES. Incorrect values can cause the child
+  process to misbehave or crash. Use the Application Verifier runtime verification
+  tool to detect invalid handles.
+
+lpProcessInformation [out]
+
+  A pointer to a PROCESS_INFORMATION structure that receives identification
+  information about the new process.
+
+  Handles in PROCESS_INFORMATION must be closed with CloseHandle when they are
+  no longer needed.
+
+Return value
+
+If the function succeeds, the return value is nonzero.
+
+If the function fails, the return value is zero. To get extended error information,
+call GetLastError.
+
+Note that the function returns before the process has finished initialization. If a
+required DLL cannot be located or fails to initialize, the process is terminated. To
+get the termination status of a process, call GetExitCodeProcess.
+Remarks
+
+The process is assigned a process identifier. The identifier is valid until the
+process terminates. It can be used to identify the process, or specified in the
+OpenProcess function to open a handle to the process. The initial thread in the
+process is also assigned a thread identifier. It can be specified in the OpenThread
+function to open a handle to the thread. The identifier is valid until the thread
+terminates and can be used to uniquely identify the thread within the system. These
+identifiers are returned in the PROCESS_INFORMATION structure.
+
+The name of the executable in the command line that the operating system provides
+to a process is not necessarily identical to that in the command line that the
+calling process gives to the CreateProcess function. The operating system may
+prepend a fully qualified path to an executable name that is provided without
+a fully qualified path.
+
+The calling thread can use the WaitForInputIdle function to wait until the new
+process has finished its initialization and is waiting for user input with no
+input pending. This can be useful for synchronization between parent and child
+processes, because CreateProcess returns without waiting for the new process to
+finish its initialization. For example, the creating process would use
+WaitForInputIdle before trying to find a window associated with the new process.
+
+The preferred way to shut down a process is by using the ExitProcess function,
+because this function sends notification of approaching termination to all DLLs
+attached to the process. Other means of shutting down a process do not notify
+the attached DLLs. Note that when a thread calls ExitProcess, other threads of
+the process are terminated without an opportunity to execute any additional code
+(including the thread termination code of attached DLLs). For more information,
+see Terminating a Process.
+
+A parent process can directly alter the environment variables of a child process
+during process creation. This is the only situation when a process can directly
+change the environment settings of another process. For more information, see
+Changing Environment Variables.
+
+If an application provides an environment block, the current directory information
+of the system drives is not automatically propagated to the new process. For
+example, there is an environment variable named =C: whose value is the current
+directory on drive C. An application must manually pass the current directory
+information to the new process. To do so, the application must explicitly create
+these environment variable strings, sort them alphabetically (because the system
+uses a sorted environment), and put them into the environment block. Typically,
+they will go at the front of the environment block, due to the environment block
+sort order.
+
+One way to obtain the current directory information for a drive X is to make the
+following call: GetFullPathName("X:", ...). That avoids an application having to
+scan the environment block. If the full path returned is X:\, there is no need to
+pass that value on as environment data, since the root directory is the default
+current directory for drive X of a new process.
+
+When a process is created with CREATE_NEW_PROCESS_GROUP specified, an implicit
+call to SetConsoleCtrlHandler(NULL,TRUE) is made on behalf of the new process;
+this means that the new process has CTRL+C disabled. This lets shells handle
+CTRL+C themselves, and selectively pass that signal on to sub-processes.
+CTRL+BREAK is not disabled, and may be used to interrupt the process/process group.
+
+Security Remarks
+
+The first parameter, lpApplicationName, can be NULL, in which case the executable
+name must be in the white space–delimited string pointed to by lpCommandLine. If
+the executable or path name has a space in it, there is a risk that a different
+executable could be run because of the way the function parses spaces. The following
+example is dangerous because the function will attempt to run "Program.exe", if it
+exists, instead of "MyApp.exe".
+
+	LPTSTR szCmdline = _tcsdup(TEXT("C:\\Program Files\\MyApp -L -S"));
+	CreateProcess(NULL, szCmdline,  ... );
+
+If a malicious user were to create an application called "Program.exe" on a system,
+any program that incorrectly calls CreateProcess using the Program Files directory
+will run this application instead of the intended application.
+
+To avoid this problem, do not pass NULL for lpApplicationName. If you do pass NULL
+for lpApplicationName, use quotation marks around the executable path in lpCommandLine.
+*/
+
 void start_process(string executable)
 {
 	if (!executable.empty()) {
 #ifdef __MINGW32__
-		char* cmd = strdup(executable.c_str());
+		static string cmdstr;
+		cmdstr.assign(executable);
 		STARTUPINFO si;
 		PROCESS_INFORMATION pi;
 		memset(&si, 0, sizeof(si));
 		si.cb = sizeof(si);
 		memset(&pi, 0, sizeof(pi));
-		if (!CreateProcess(NULL, cmd, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi))
+LOG_INFO("Starting external process: %s", cmdstr.c_str());
+		if (!CreateProcess(	NULL, const_cast<char*>(cmdstr.c_str()),
+							NULL, NULL,
+							FALSE, CREATE_NO_WINDOW,
+							NULL, NULL,
+							&si, &pi))
 			LOG_ERROR("CreateProcess failed with error code %ld", GetLastError());
+		MilliSleep(100);
 		CloseHandle(pi.hProcess);
 		CloseHandle(pi.hThread);
-		free(cmd);
+LOG_INFO("Process handles closed");
 #else
 #ifdef __APPLE__
 	if (executable.find(".app") == (executable.length() - 4)) {
@@ -366,6 +672,7 @@ void delayed_startup(void *)
 	n3fjp_init();
 	arq_init();
 	FD_init();
+	DXcluster_init();
 
 	start_psm_thread();
 
@@ -383,6 +690,11 @@ void delayed_startup(void *)
 
 	auto_start();
 
+	if (progStatus.WK_online) {
+		btn_WK_connect->value(1);
+		WK_connect(1);
+	}
+
 	if (progdefaults.check_for_updates)
 		cb_mnuCheckUpdate((Fl_Widget *)0, NULL);
 
@@ -397,7 +709,8 @@ int main(int argc, char ** argv)
 	program_start_time = time(0);
 
 	// ztimer must be run by FLTK's timeout handler
-	ztimer((void*)true);
+	TOD_init(); // initialize time of day thread
+//	ztimer((void*)true);
 
 	active_modem = new NULLMODEM;
 
@@ -486,6 +799,10 @@ int main(int argc, char ** argv)
 				cbq[i]->attach(i, "TRX_TID");
 				break;
 
+			case TOD_TID:
+				cbq[i]->attach(i, "TOD_TID");
+				break;
+
 			case QRZ_TID:
 				cbq[i]->attach(i, "QRZ_TID");
 				break;
@@ -506,6 +823,10 @@ int main(int argc, char ** argv)
 				cbq[i]->attach(i, "ADIF_RW_TID");
 				break;
 
+			case ADIF_MERGE_TID:
+				cbq[i]->attach(i, "ADIF_MERGE_TID");
+				break;
+
 			case XMLRPC_TID:
 				cbq[i]->attach(i, "XMLRPC_TID");
 				break;
@@ -542,6 +863,14 @@ int main(int argc, char ** argv)
 				cbq[i]->attach(i, "N3FJP_TID");
 				break;
 
+			case DXCC_TID:
+				cbq[i]->attach(i, "DXCC_TID");
+				break;
+
+			case WKEY_TID:
+				cbq[i]->attach(i, "WKEY_TID");
+				break;
+
 			case FLMAIN_TID:
 				cbq[i]->attach(i, "FLMAIN_TID");
 				break;
@@ -634,6 +963,7 @@ int main(int argc, char ** argv)
 	LOG_INFO("WrapDir: %s", WrapDir.c_str());
 	LOG_INFO("TalkDir: %s", TalkDir.c_str());
 	LOG_INFO("TempDir: %s", TempDir.c_str());
+	LOG_INFO("LoTWDir: %s", LoTWDir.c_str());
 	LOG_INFO("KmlDir: %s", KmlDir.c_str());
 	LOG_INFO("PskMailDir: %s", PskMailDir.c_str());
 
@@ -713,6 +1043,7 @@ int main(int argc, char ** argv)
 			 progdefaults.slowcpu ? "true" : "false",
 			 src_get_name(progdefaults.sample_converter));
 	}
+
 	if (progdefaults.XmlRigFilename.empty())
 		progdefaults.XmlRigFilename = xmlfname;
 
@@ -758,6 +1089,10 @@ int main(int argc, char ** argv)
 
 	dlgViewer = createViewer();
 	create_logbook_dialogs();
+	dxcluster_viewer = dxc_window();
+
+	dxcluster_viewer->hide();
+
 	LOGBOOK_colors_font();
 
 	if( progdefaults.kml_save_dir.empty() ) {
@@ -804,6 +1139,8 @@ void exit_process() {
 	stop_psm_thread();
 	arq_close();
 	FD_close();
+	DXcluster_close();
+
 	kiss_close(false);
 	maclogger_close();
 	XML_RPC_Server::stop();
@@ -1543,14 +1880,18 @@ void set_platform_ui(void)
 #define OUT_RATE 8000
 double speed_test(int converter, unsigned repeat)
 {
+	float input_frames[IN_RATE];
+	float output_frames[OUT_RATE];
+
 	SRC_DATA src;
+
 	src.src_ratio = (double)OUT_RATE / IN_RATE;
 	src.input_frames = IN_RATE;
 	src.output_frames = OUT_RATE;
-	src.data_in = new float[src.input_frames];
-	src.data_out = new float[src.output_frames];
+	src.data_in = &input_frames[0];
+	src.data_out = &output_frames[0];
 
-	memset(src.data_in, 0, src.input_frames * sizeof(float));
+	memset(input_frames, 0, sizeof(input_frames));
 
 	// warm up
 	src_simple(&src, converter, 1);
@@ -1569,9 +1910,6 @@ double speed_test(int converter, unsigned repeat)
 	clock_gettime(CLOCK_REALTIME, &t1);
 #endif
 
-	delete [] src.data_in;
-	delete [] src.data_out;
-
 	t0 = t1 - t0;
 	return repeat / (t0.tv_sec + t0.tv_nsec/1e9);
 }
@@ -1645,6 +1983,7 @@ static void checkdirectories(void)
 		{ WrapDir, "wrap", 0 },
 		{ TalkDir, "talk", 0 },
 		{ TempDir, "temp", 0 },
+		{ LoTWDir, "LOTW", 0 },
 		{ KmlDir, "kml", 0 },
 		{ DATA_dir, "data", 0 },
 	};
