@@ -3,7 +3,7 @@
 //
 // support for ARQ server/client system such as pskmail and fl_arq
 //
-// Copyright (C) 2006-2013
+// Copyright (C) 2006-2017
 //		Dave Freese, W1HKJ
 // Copyright (C) 2008-2013
 //		Stelios Bounanos, M0GLD
@@ -157,61 +157,52 @@ static void set_button(Fl_Button* button, bool value)
 
 void ParseMode(string src)
 {
-	if (src.find("XMTTUNE") != string::npos) {
+LOG_INFO("%s", src.c_str());
+	if ((src.find("XMTTUNE") != string::npos) ||
+		(src.find("PTTTUNE") != string::npos)) {
 		int msecs = 100;
 		if (src.length() > 7) {
 			int ret = sscanf( src.substr(7, src.length() - 7).c_str(), "%d", &msecs);
 			if (ret != 1 || msecs < 10 || msecs > 20000) msecs = 100;
 		}
-		if (debug_pskmail)
-			LOG_INFO("%s %5.2f sec", "ARQ tune on", msecs/1000.0);
-		trx_tune();
+//		if (debug_pskmail)
+			LOG_INFO("%s %5.2f sec", "ARQ set ptt-tune on", msecs/1000.0);
+		REQ_SYNC(&waterfall::set_XmtRcvBtn, wf, true);
+		REQ_SYNC(trx_tune);
 		MilliSleep(msecs);
-		if (debug_pskmail)
-			LOG_INFO("%s", "ARQ tune off");
-		trx_receive();
-		return;
+//		if (debug_pskmail)
+			LOG_INFO("%s", "ARQ set ptt-tune off");
+		REQ_SYNC(&waterfall::set_XmtRcvBtn, wf, false);
+		REQ_SYNC(trx_receive);
 	}
-	if (src.find("PTTTUNE") != string::npos) {
-		int msecs = 100;
-		if (src.length() > 7) {
-			int ret = sscanf( src.substr(7, src.length() - 7).c_str(), "%d", &msecs);
-			if (ret != 1 || msecs < 10 || msecs > 20000) msecs = 100;
-		}
-		if (debug_pskmail)
-			LOG_INFO("%s %5.2f sec", "ARQ set ptt on", msecs/1000.0);
-		push2talk->set(true);
-		REQ(&waterfall::set_XmtRcvBtn, wf, true);
-		MilliSleep(msecs);
-		if (debug_pskmail)
-			LOG_INFO("%s", "ARQ set ptt off");
-		push2talk->set(false);
-		REQ(&waterfall::set_XmtRcvBtn, wf, false);
-		return;
-	}
-	for (size_t i = 0; i < NUM_MODES; ++i) {
+	else for (size_t i = 0; i < NUM_MODES; ++i) {
 		if (strlen(mode_info[i].pskmail_name) > 0) {
 			if (src == mode_info[i].pskmail_name) {
-				REQ_SYNC(init_modem_sync, mode_info[i].mode, 0);
-				AbortARQ();
-				WriteARQ('\002');
-				if (debug_pskmail)
+				if (active_modem->get_mode() == mode_info[i].mode) {
+					LOG_INFO("Active modem already set to %s", src.c_str());
+				} else {
+					REQ_SYNC(init_modem_sync, mode_info[i].mode, 0);
+					MilliSleep(100);
+//				AbortARQ();
+//				if (debug_pskmail)
 					LOG_INFO("Modem set to %s", mode_info[i].pskmail_name);
+				}
 				break;
 			}
 		}
 	}
+	WriteARQ('\002');
 }
 
 void ParseRSID(string src)
 {
 	if (src == "ON") {
-		if (debug_pskmail)
+//		if (debug_pskmail)
 			LOG_INFO("%s", "RsID turned ON");
 		REQ(set_button, btnRSID, 1);
 	}
 	if (src == "OFF") {
-		if (debug_pskmail)
+//		if (debug_pskmail)
 			LOG_INFO("%s", "RsID turned OFF");
 		REQ(set_button, btnRSID, 0);
 	}
@@ -221,12 +212,12 @@ void ParseRSID(string src)
 void ParseTxRSID(string src)
 {
 	if (src == "ON") {
-		if (debug_pskmail)
+//		if (debug_pskmail)
 			LOG_INFO("%s", "TxRsID turned ON");
 		REQ(set_button, btnTxRSID, 1);
 	}
 	if (src == "OFF") {
-		if (debug_pskmail)
+//		if (debug_pskmail)
 			LOG_INFO("%s", "TxRsID turned OFF");
 		REQ(set_button, btnTxRSID, 0);
 	}
@@ -240,11 +231,14 @@ void parse_arqtext(string &toparse)
 
 	if (toparse.empty()) return;
 
+LOG_INFO("parsing: %s", noctrl(toparse).c_str());
+
+
 	idxCmd = toparse.find("<cmd>");
 	idxCmdEnd = toparse.find("</cmd>");
 
 	while ( idxCmd != string::npos && idxCmdEnd != string::npos && idxCmdEnd > idxCmd ) {
-		LOG_INFO("Parsing: %s", noctrl(toparse.substr(idxCmd, idxCmdEnd - idxCmd + 6)).c_str());
+//		LOG_INFO("Parsing: %s", noctrl(toparse.substr(idxCmd, idxCmdEnd - idxCmd + 6)).c_str());
 
 		strCmdText = toparse.substr(idxCmd + 5, idxCmdEnd - idxCmd - 5);
 		if (strCmdText == "server" && mailserver == false && mailclient == false) {
@@ -286,8 +280,8 @@ void parse_arqtext(string &toparse)
 			if (	idxSubCmdEnd != string::npos &&
 				idxSubCmdEnd > idxSubCmd ) {
 				strSubCmd = strCmdText.substr(idxSubCmd + 6, idxSubCmdEnd - idxSubCmd - 6);
-				ParseMode(strSubCmd);
 				LOG_INFO("%s %s", "ARQ mode ", strSubCmd.c_str());
+				ParseMode(strSubCmd);
 			}
 		} else if ((idxSubCmd = strCmdText.find("<rsid>")) != string::npos) {
 			idxSubCmdEnd = strCmdText.find("</rsid>");
@@ -316,6 +310,8 @@ void parse_arqtext(string &toparse)
 		idxCmd = toparse.find("<cmd>");
 		idxCmdEnd = toparse.find("</cmd>");
 	}
+	if (!toparse.empty())
+		LOG_INFO("Remaining text: %s", noctrl(toparse).c_str());
 }
 
 #define TIMEOUT 180 // 3 minutes
@@ -472,7 +468,7 @@ static string errstring;
 
 static pthread_t* arq_socket_thread = 0;
 ARQ_SOCKET_Server* ARQ_SOCKET_Server::inst = 0;
-static std::vector<ARQCLIENT> arqclient; // Protected by arq_mutex
+static std::vector<ARQCLIENT *> arqclient; // Protected by arq_mutex
 
 void arq_run(Socket);
 
@@ -526,10 +522,21 @@ bool ARQ_SOCKET_Server::start(const char* node, const char* service)
 	return !pthread_create(arq_socket_thread, NULL, thread_func, NULL);
 }
 
+bool server_stopped = false;
+
 void ARQ_SOCKET_Server::stop(void)
 {
 	if (!inst)
 		return;
+	inst->run = false;
+
+	int timeout = 20;
+	SET_THREAD_CANCEL();
+	while(!server_stopped) {
+		MilliSleep(50);
+		if (--timeout == 0) break;
+	}
+
 	delete inst;
 	inst = 0;
 }
@@ -544,7 +551,7 @@ void* ARQ_SOCKET_Server::thread_func(void*)
 	// On woe32 we block for a short time and test for cancellation.
 	while (inst->run) {
 		try {
-#ifdef __WOE32__
+#ifdef __WOE32__ 
 			if (inst->server_socket->wait(0))
 				arq_run(inst->server_socket->accept());
 #else
@@ -569,19 +576,17 @@ void* ARQ_SOCKET_Server::thread_func(void*)
 		guard_lock arq_lock(&arq_mutex);
 
 		if (!arqclient.empty()) {
-			vector<ARQCLIENT>::iterator p = arqclient.begin();
-			while (p != arqclient.end()) {
-				try {
-					(*p).sock.close();
-					arqclient.erase(p);
-				}
-				catch (...) {;}
-				p++;
+			for (vector<ARQCLIENT *>::iterator p = arqclient.begin();
+					p < arqclient.end();
+					p++) {
+				(*p)->sock.close();
+				arqclient.erase(p);
 			}
 		}
 	}
 
 	inst->server_socket->close();
+server_stopped = true;
 	return NULL;
 }
 
@@ -602,16 +607,16 @@ void arq_run(Socket s)
 	struct timeval t = { 0, 20000 };
 	s.set_timeout(t);
 	s.set_nonblocking();
-	ARQCLIENT client;
-	client.sock = s;
-	client.keep_alive = time(0);
+	ARQCLIENT *client = new ARQCLIENT;
+	client->sock = s;
+	client->keep_alive = time(0);
 	arqclient.push_back(client);
 	arqmode = true;
-	vector<ARQCLIENT>::iterator p = arqclient.begin();
+	vector<ARQCLIENT *>::iterator p = arqclient.begin();
 	ostringstream outs;
 	outs << "Clients: ";
 	while (p != arqclient.end()) {
-		outs << (*p).sock.fd() << " ";
+		outs << (*p)->sock.fd() << " ";
 		p++;
 	}
 	LOG_INFO("%s", outs.str().c_str());
@@ -630,20 +635,20 @@ void WriteARQsocket(unsigned char* data, size_t len)
 		outs += asc[data[i] & 0x7F];
 	LOG_INFO("%s", outs.c_str());
 
-	vector<ARQCLIENT>::iterator p;
+	vector<ARQCLIENT *>::iterator p;
 	for (p = arqclient.begin(); p < arqclient.end(); p++) {
 		try {
-			(*p).sock.wait(1);
-			(*p).sock.send(data, len);
-			(*p).keep_alive = time(0);
+			(*p)->sock.wait(1);
+			(*p)->sock.send(data, len);
+			(*p)->keep_alive = time(0);
 			p++;
 		}
 		catch (const SocketException& e) {
-			LOG_INFO("closing socket fd %d %s", (*p).sock.fd(), e.what());
+			LOG_INFO("closing socket fd %d %s", (*p)->sock.fd(), e.what());
 			try {
-				(*p).sock.close();
+				(*p)->sock.close();
 			} catch (const SocketException& e) {
-				LOG_ERROR("Socket error on # %d, %d: %s", (*p).sock.fd(), e.error(), e.what());
+				LOG_ERROR("Socket error on # %d, %d: %s", (*p)->sock.fd(), e.error(), e.what());
 			}
 			arqclient.erase(p);
 		}
@@ -659,23 +664,23 @@ void test_arq_clients()
 	if (arqclient.empty()) return;
 	static string instr;
 	instr.clear();
-	vector<ARQCLIENT>::iterator p;
+	vector<ARQCLIENT *>::iterator p;
 	p = arqclient.begin();
 	time_t now;
 	while (p != arqclient.end()) {
-		if (difftime(now = time(0), (*p).keep_alive) > CLIENT_TIMEOUT) {
+		if (difftime(now = time(0), (*p)->keep_alive) > CLIENT_TIMEOUT) {
 			try {
-				(*p).sock.wait(1);
-				(*p).sock.send("\0", 1);
-				(*p).keep_alive = now;
+				(*p)->sock.wait(1);
+				(*p)->sock.send("\0", 1);
+				(*p)->keep_alive = now;
 				p++;
 			}
 			catch (const SocketException& e) {
-				LOG_INFO("socket %d timed out, error %d, %s", (*p).sock.fd(), e.error(), e.what());
+				LOG_INFO("socket %d timed out, error %d, %s", (*p)->sock.fd(), e.error(), e.what());
 				try {
-					(*p).sock.close();
+					(*p)->sock.close();
 				} catch (const SocketException& e) {
-					LOG_ERROR("Socket error on # %d, %d: %s", (*p).sock.fd(), e.error(), e.what());
+					LOG_ERROR("Socket error on # %d, %d: %s", (*p)->sock.fd(), e.error(), e.what());
 				}
 				arqclient.erase(p);
 			}
@@ -694,27 +699,27 @@ bool Socket_arqRx()
 		if (arqclient.empty()) return false;
 
 		static string instr;
-		vector<ARQCLIENT>::iterator p = arqclient.begin();
+		vector<ARQCLIENT *>::iterator p = arqclient.begin();
 		size_t n = 0;
 		instr.clear();
 
 		while (p != arqclient.end()) {
 			try {
-				(*p).sock.wait(0);
-				n = (*p).sock.recv(instr);
+				(*p)->sock.wait(0);
+				n = (*p)->sock.recv(instr);
 				if ( n > 0) {
 					txstring.append(instr);
-					(*p).keep_alive = time(0);
+					(*p)->keep_alive = time(0);
 				}
 				p++;
 			}
 			catch (const SocketException& e) {
 				txstring.clear();
-				LOG_INFO("closing socket fd %d, %d: %s", (*p).sock.fd(), e.error(), e.what());
+				LOG_INFO("closing socket fd %d, %d: %s", (*p)->sock.fd(), e.error(), e.what());
 				try {
-					(*p).sock.close();
+					(*p)->sock.close();
 				} catch (const SocketException& e) {
-					LOG_ERROR("socket error on # %d, %d: %s", (*p).sock.fd(), e.error(), e.what());
+					LOG_ERROR("socket error on # %d, %d: %s", (*p)->sock.fd(), e.error(), e.what());
 				}
 				arqclient.erase(p);
 			}
@@ -750,7 +755,7 @@ bool Socket_arqRx()
 		} else {
 			arqtext.append(txstring);
 			if (trx_state != STATE_TX) {
-				if (debug_pskmail)
+//				if (debug_pskmail)
 					LOG_INFO("%s","Restarting TX");
 				start_tx();
 			}
@@ -802,8 +807,10 @@ static void *arq_loop(void *args)
 
 	for (;;) {
 		/* see if we are being canceled */
-		if (arq_exit)
+		if (arq_exit) {
+printf("exiting arq_loop\n");
 			break;
+}
 
 		test_arq_clients();
 
@@ -843,6 +850,7 @@ void arq_init()
 	arq_enabled = false;
 
 	txstring.clear();
+	arqclient.clear();
 
 	if (!ARQ_SOCKET_Server::start( progdefaults.arq_address.c_str(), progdefaults.arq_port.c_str() )) {
 		arq_enabled = false;
@@ -868,6 +876,7 @@ void arq_close(void)
 	arq_exit = true;
 
 	// and then wait for it to die
+printf("pthread_join arq_thread\n");
 	pthread_join(arq_thread, NULL);
 	arq_enabled = false;
 	LOG_INFO("ARQ closed");
