@@ -529,8 +529,6 @@ int configuration::setDefaults()
 
 	inpTTYdev->value(PTTdev.c_str());
 
-	inpGPIOPort->value(GPIOPort);
-
 	chkUSEHAMLIB->value(0);
 	chkUSERIGCAT->value(0);
 	chkUSEXMLRPC->value(0);
@@ -734,8 +732,6 @@ void configuration::initInterface()
 
 	PTTdev = inpTTYdev->value();
 
-	GPIOPort = static_cast<int>(inpGPIOPort->value());
-
 #if USE_HAMLIB
 	chkUSEHAMLIBis = chkUSEHAMLIB->value();
      HamlibCMDptt = btnHamlibCMDptt->value();
@@ -775,6 +771,7 @@ void configuration::initInterface()
 	} else if (chkUSEHAMLIBis) { // start the hamlib thread
 		if (hamlib_init(HamlibCMDptt)) {
 			LOG_INFO("%s", "using HAMLIB xcvr control");
+			btnInitHAMLIB->deactivate();
 			wf->USB(true);
 			wf->setQSY(1);
 		} else {
@@ -804,8 +801,6 @@ void configuration::initInterface()
 		push2talk->reset(PTT::PTT_TTY);
 	else if (UsePPortPTT)
 		push2talk->reset(PTT::PTT_PARPORT);
-	else if (UseGPIOPTT)
-		push2talk->reset(PTT::PTT_GPIO);
 	else if (UseUHrouterPTT)
 		push2talk->reset(PTT::PTT_UHROUTER);
 	else
@@ -913,6 +908,8 @@ void configuration::testCommPorts()
 		inpRIGdev->add(gbuf.gl_pathv[j]);
 #  endif
 		inpXmlRigDevice->add(gbuf.gl_pathv[j]);
+
+		select_WK_CommPort->add(gbuf.gl_pathv[j]);
 	}
 	globfree(&gbuf);
 #endif
@@ -941,6 +938,8 @@ void configuration::testCommPorts()
 			inpRIGdev->add(ttyname);
 #  endif
 			inpXmlRigDevice->add(ttyname);
+
+			select_WK_CommPort->add(ttyname);
 		}
 #else // __APPLE__
 		glob_t gbuf;
@@ -964,6 +963,7 @@ void configuration::testCommPorts()
 #  endif
 			inpXmlRigDevice->add(gbuf.gl_pathv[j]);
 
+			select_WK_CommPort->add(gbuf.gl_pathv[j]);
 		}
 		globfree(&gbuf);
 #endif // __APPLE__
