@@ -230,8 +230,11 @@ Address::Address(const char* host, int port, const char* proto_name)
 	ostringstream s;
 	s << port;
 	service = s.str();
-
-	lookup(proto_name);
+	try {
+		lookup(proto_name);
+	} catch (...) {
+		throw;
+	}
 }
 
 Address::Address(const char* host, const char* port_name, const char* proto_name)
@@ -243,8 +246,11 @@ Address::Address(const char* host, const char* port_name, const char* proto_name
 	memset(&host_entry, 0, sizeof(host_entry));
 	memset(&service_entry, 0, sizeof(service_entry));
 #endif
-
-	lookup(proto_name);
+	try {
+		lookup(proto_name);
+	} catch (...) {
+		throw;
+	}
 }
 
 Address::Address(const Address& addr)
@@ -326,9 +332,9 @@ void Address::lookup(const char* proto_name)
 	if (service.find_first_not_of("0123456789") == string::npos)
 		hints.ai_flags |= AI_NUMERICSERV;
 
-	int r;
-	if ((r = getaddrinfo(node.empty() ? NULL : node.c_str(), service.c_str(), &hints, &info)) < 0)
-		throw SocketException(r, "getaddrinfo");
+	int r = getaddrinfo(node.empty() ? NULL : node.c_str(), service.c_str(), &hints, &info);
+	if (r != 0)
+		throw SocketException(gai_strerror(r));
 
 #else // use gethostbyname etc.
 	memset(&host_entry, 0, sizeof(host_entry));
@@ -339,6 +345,7 @@ void Address::lookup(const char* proto_name)
 	struct hostent* hp;
 	if ((hp = gethostbyname(node.c_str())) == NULL)
 		throw SocketException(hstrerror(HOST_NOT_FOUND));
+
 	copy_hostent(&host_entry, hp);
 
 	int port;
@@ -600,12 +607,13 @@ void Socket::open(const Address& addr)
 	address = addr;
 	size_t n = address.size();
 
-	for (anum = 0; anum < n; anum++) {
-		ainfo = address.get(anum);
-#ifndef NDEBUG
-		LOG_DEBUG("Trying %s", address.get_str(ainfo).c_str());
+#ifdef __FreeBSD__
+	for (anum = n-1; anum >= 1; anum--) {
+#else
+	for (anum = n-1; anum >= 0; anum--) {
 #endif
-
+		ainfo = address.get(anum);
+		LOG_INFO("Trying %s", address.get_str(ainfo).c_str());
 		if ((sockfd = socket(ainfo->ai_family, ainfo->ai_socktype, ainfo->ai_protocol)) != -1)
 			break;
 	}
@@ -854,20 +862,26 @@ Socket * Socket::accept2(void)
 void Socket::connect(void)
 {
     connected_flag = false;
-#ifndef NDEBUG
-	LOG_DEBUG("Connecting to %s", address.get_str(ainfo).c_str());
-#endif
-	LOG_INFO("Connecting to %s", address.get_str(ainfo).c_str());
 	int res = ::connect(sockfd, ainfo->ai_addr, ainfo->ai_addrlen);
 	if (res == -1) {
-		LOG_INFO("Response %d, error %d", res, errno);
-		if ( (errno == EAGAIN) || (errno == EINPROGRESS) )
-			throw SocketException(errno, "connect");
-		if (errno == EALREADY) {
+		if (errno == 0) {
+			LOG_INFO("Connected to %s", address.get_str(ainfo).c_str());
 			connected_flag = true;
 			return;
 		}
+		if (!errno || (errno == EWOULDBLOCK) || (errno == EINPROGRESS) || 
+			(errno == EISCONN) || (errno == EALREADY) ) {
+			LOG_DEBUG("Connect attempt to %s : %d, %s", 
+				address.get_str(ainfo).c_str(),
+				errno, strerror(errno));
+			return;
+		}
+		LOG_DEBUG("Connect to %s failed: %d, %s", 
+			address.get_str(ainfo).c_str(),
+			errno, strerror(errno));
+		throw SocketException(errno, "connect");
 	}
+	LOG_INFO("Connected to %s", address.get_str(ainfo).c_str());
     connected_flag = true;
 }
 ///
@@ -1120,7 +1134,11 @@ size_t Socket::sendTo(const void* buf, size_t len)
 ///
 size_t Socket::sendTo(const std::string& buf)
 {
-	return sendTo(buf.data(), buf.length());
+	try {
+		return sendTo(buf.data(), buf.length());
+	} catch (...) {
+		throw;
+	}
 }
 
 //
