@@ -44,7 +44,7 @@
 
 #include "threads.h"
 
-class XmlRpcImpl;
+struct XmlRpcImpl;
 
 #include "globals.h"
 #include "configuration.h"
@@ -198,7 +198,6 @@ struct XmlRpcImpl : public XmlRpcServer
 	{
 		exit();
 		shutdown();
-		LOG_INFO("XMLRPC server loop terminated");
 	}
 };
 
