@@ -46,11 +46,13 @@
 const char *sztid[] = {
 	"INVALID_TID",
 	"TRX_TID",
+	"TOD_TID",
 	"QRZ_TID",
 	"RIGCTL_TID",
 	"NORIGCTL_TID",
 	"EQSL_TID",
 	"ADIF_RW_TID",
+	"ADIF_MERGE_TID",
 	"XMLRPC_TID",
 	"ARQ_TID",
 	"ARQSOCKET_TID",
@@ -60,6 +62,8 @@ const char *sztid[] = {
 	"PSM_TID",
 	"FD_TID",
 	"N3FJP_TID",
+	"DXCC_TID",
+	"WKEY_TID",
 	"FLMAIN_TID"
 };
 
