@@ -50,6 +50,9 @@
 
 #include "gettext.h"
 
+extern void n3fjp_set_freq(long);
+extern bool n3fjp_connected;
+
 LOG_FILE_SOURCE(debug::LOG_RIGCONTROL);
 
 using namespace std;
@@ -396,12 +399,15 @@ void sendFreq(long int f)
 		set_flrig_freq(f);
 #if USE_HAMLIB
 	else if (progdefaults.chkUSEHAMLIBis)
-			hamlib_setfreq(f);
+		hamlib_setfreq(f);
 #endif
 	else if (progdefaults.chkUSERIGCATis)
 		rigCAT_setfreq(f);
 	else
 		noCAT_setfreq(f);
+
+	if (n3fjp_connected)
+		n3fjp_set_freq(f);
 }
 
 void qso_movFreq(Fl_Widget* w, void *data)
@@ -517,10 +523,6 @@ void qso_updateEntry(int i, std::string usage)
 
 void setTitle()
 {
-	if (windowTitle.empty()) return;
-	size_t p = main_window_title.find(" / ");
-	if (p != string::npos) main_window_title.erase(p);
-	main_window_title.append(" / ").append(windowTitle);
 	update_main_title();
 }
 
@@ -528,7 +530,7 @@ bool init_Xml_RigDialog()
 {
 	LOG_DEBUG("xml rig");
 	initOptionMenus();
-	windowTitle = xmlrig.rigTitle;
+	xcvr_title = xmlrig.rigTitle;
 	setTitle();
 	return true;
 }
@@ -536,12 +538,16 @@ bool init_Xml_RigDialog()
 bool init_NoRig_RigDialog()
 {
 	LOG_DEBUG("no rig");
+	qso_opBW->clear();
+	qso_opBW->add("  ");
+	qso_opBW->index(0);
+	qso_opBW->redraw();
 	qso_opBW->deactivate();
 	qso_opMODE->clear();
 
-printf("init_NoRig_RigDialog()\n");
+//printf("init_NoRig_RigDialog()\n");
 	for (size_t i = 0; i < sizeof(modes)/sizeof(modes[0]); i++) {
-printf("adding %s\n", modes[i].name);
+//printf("adding %s\n", modes[i].name);
 		qso_opMODE->add(modes[i].name);
 	}
 // list of LSB type modes that various xcvrs report via flrig
@@ -567,7 +573,7 @@ printf("adding %s\n", modes[i].name);
 	qso_opMODE->index(3);
 	qso_opMODE->activate();
 
-	windowTitle.clear();
+	xcvr_title.clear();
 	setTitle();
 
 	return true;
@@ -587,8 +593,8 @@ bool init_Hamlib_RigDialog()
 		qso_opMODE->add(modes[i].name);
 	}
 
-	windowTitle = "Hamlib ";
-	windowTitle.append(xcvr->getName());
+	xcvr_title = "Hamlib ";
+	xcvr_title.append(xcvr->getName());
 
 	setTitle();
 
