@@ -44,6 +44,7 @@
 #include "confdialog.h"
 
 #include "icons.h"
+#include "fl_digi.h"
 
 using namespace std;
 
@@ -62,9 +63,13 @@ void parseMODEREPLY(size_t &);
 void parseTITLE(size_t &);
 void parseLSBMODES(size_t &);
 void parseDISCARD(size_t &);
+void parseDEBUG(size_t &);
+void parseNOSERIAL(size_t &);
+void parseASCII(size_t &);
 
 void parseWRITE_DELAY(size_t &);
 void parseINIT_DELAY(size_t &);
+void parseWAIT_FOR_DEVICE(size_t &);
 void parsePOST_WRITE_DELAY(size_t &);
 void parseRETRIES(size_t &);
 void parseTIMEOUT(size_t &);
@@ -79,6 +84,12 @@ void parseDTRPTT(size_t &);
 void parseRESTORE_TIO(size_t &);
 void parseECHO(size_t &);
 void parseVSP(size_t &);
+void parseLOGSTR(size_t &);
+void parsePOLLINT(size_t &);
+void parseSMETER(size_t &);
+void parsePMETER(size_t &);
+void parseNOTCH(size_t &);
+void parsePWRLEVEL(size_t &);
 
 void parseIOSsymbol(size_t &);
 void parseIOSsize(size_t &);
@@ -145,11 +156,21 @@ TAGS rigdeftags[] = {
 	{"<DTRPLUS", parseDTRPLUS},
 	{"<RTSPTT", parseRTSPTT},
 	{"<DTRPTT", parseDTRPTT},
+	{"<WAIT_FOR_DEVICE", parseWAIT_FOR_DEVICE},
 	{"<RESTORE_TIO", parseRESTORE_TIO},
 	{"<ECHO", parseECHO},
 	{"<CMDPTT", parseCMDPTT},
 	{"<STOPBITS", parseSTOPBITS},
 	{"<VSP", parseVSP},
+	{"<LOGSTR", parseLOGSTR},
+	{"<POLLINT", parsePOLLINT},
+	{"<SMETER", parseSMETER},
+	{"<PMETER", parsePMETER},
+	{"<NOTCH", parseNOTCH},
+	{"<PWRLEVEL", parsePWRLEVEL},
+	{"<DEBUG", parseDEBUG},
+	{"<NOSERIAL", parseNOSERIAL},
+	{"<ASCII", parseASCII},
 	{0, 0}
 };
 
@@ -519,6 +540,42 @@ void parseRIG(size_t &p0)
 	p0 = pend;
 }
 
+//---------------------------------------------------------------------
+// Parse DEBUG, use during xml file creation/testing
+//---------------------------------------------------------------------
+
+void parseDEBUG(size_t &p0){
+	bool val = getBool(p0);
+	xmlrig.debug = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+//---------------------------------------------------------------------
+// Parse NOSERIAL, use during xml file creation/testing
+// suppresses serial port i/o
+//---------------------------------------------------------------------
+
+void parseNOSERIAL(size_t &p0){
+	bool val = getBool(p0);
+	xmlrig.noserial = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+//---------------------------------------------------------------------
+// Parse ASCII, use during xml file creation/testing
+// prints events as ASCII string vice HEX sequence
+//---------------------------------------------------------------------
+
+void parseASCII(size_t &p0){
+	bool val = getBool(p0);
+	xmlrig.ascii = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+
 //---------------------------------------------------------------------
 // Parse Baudrate, write_delay, post_write_delay, timeout, retries
 // RTSCTS handshake
@@ -554,6 +611,13 @@ void parseINIT_DELAY(size_t &p0){
 	p0 = pend;
 }
 
+void parseWAIT_FOR_DEVICE(size_t &p0){
+	int val = getInt(p0);
+	xmlrig.wait_for_device = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
 void parsePOST_WRITE_DELAY(size_t &p0){
 	int val = getInt(p0);
 	xmlrig.post_write_delay = val;
@@ -644,6 +708,110 @@ void parseVSP(size_t &p0)
 	p0 = pend;
 }
 
+void parseLOGSTR(size_t &p0) {
+	bool val = getBool(p0);
+	xmlrig.logstr = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+void parsePOLLINT(size_t &p0) {
+	int val = getInt(p0);
+	if (val < 100) val = 1000;
+	if (val > 20000) val = 20000;
+	xmlrig.pollinterval = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+void parseSMETER(size_t &p0) {
+	string strmeter = getElement(p0);
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+	xmlrig.smeter.clear();
+	int val, sm;
+	size_t p = strmeter.find(",");
+	while ( !strmeter.empty() && (p != string::npos) ) {
+		val = atoi(&strmeter[0]);
+		sm = atoi(&strmeter[p+1]);
+		xmlrig.smeter.push_back(PAIR(val,sm));
+		p = strmeter.find(";");
+		if (p == string::npos) strmeter.clear();
+		else {
+			strmeter.erase(0, p+1);
+			p = strmeter.find(",");
+		}
+	}
+	xmlrig.use_smeter = true;
+}
+
+void parsePMETER(size_t &p0) {
+	string strmeter = getElement(p0);
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+	xmlrig.smeter.clear();
+	int val, sm;
+	size_t p = strmeter.find(",");
+	while ( !strmeter.empty() && (p != string::npos) ) {
+		val = atoi(&strmeter[0]);
+		sm = atoi(&strmeter[p+1]);
+		xmlrig.pmeter.push_back(PAIR(val,sm));
+		p = strmeter.find(";");
+		if (p == string::npos) strmeter.clear();
+		else {
+			strmeter.erase(0, p+1);
+			p = strmeter.find(",");
+		}
+	}
+	xmlrig.use_pwrmeter = true;
+}
+
+void parseNOTCH(size_t &p0) {
+	string strnotch = getElement(p0);
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+	xmlrig.notch.clear();
+	int val, ntch;
+	size_t p = strnotch.find(",");
+	while ( !strnotch.empty() && (p != string::npos) ) {
+		val = atoi(&strnotch[0]);
+		ntch = atoi(&strnotch[p+1]);
+		xmlrig.notch.push_back(PAIR(val,ntch));
+		p = strnotch.find(";");
+		if (p == string::npos) strnotch.clear();
+		else {
+			strnotch.erase(0, p+1);
+			p = strnotch.find(",");
+		}
+	}
+	xmlrig.use_notch = true;
+}
+
+void parsePWRLEVEL(size_t &p0) {
+	string strpwrlevel = getElement(p0);
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+	xmlrig.pwrlevel.clear();
+	int val, pwr;
+	float min = 500, max = 0;
+	size_t p = strpwrlevel.find(",");
+	while ( !strpwrlevel.empty() && (p != string::npos) ) {
+		val = atoi(&strpwrlevel[0]);
+		pwr = atoi(&strpwrlevel[p+1]);
+		if (pwr < min) min = pwr;
+		if (pwr > max) max = pwr;
+		xmlrig.pwrlevel.push_back(PAIR(val,pwr));
+		p = strpwrlevel.find(";");
+		if (p == string::npos) strpwrlevel.clear();
+		else {
+			strpwrlevel.erase(0, p+1);
+			p = strpwrlevel.find(",");
+		}
+	}
+	pwr_level->range(min, max);
+	xmlrig.use_pwrlevel = true;
+}
+
 //---------------------------------------------------------------------
 // Parse IOS (serial stream format) definitions
 //---------------------------------------------------------------------
@@ -933,6 +1101,7 @@ bool readRigXML()
 		xmlfile.close();
 		if (testXML()) {
 			parseXML();
+			xmlrig.xmlok = true;
 			return true;
 		}
 	}
@@ -944,11 +1113,12 @@ void selectRigXmlFilename()
 	string deffilename;
 	deffilename = progdefaults.XmlRigFilename;
 	const char *p = FSEL::select(_("Open rig xml file"), _("Fldigi rig xml definition file\t*.xml"), deffilename.c_str());
-	if (p) {
-		progdefaults.XmlRigFilename = p;
-		txtXmlRigFilename->value(fl_filename_name(p));
-		loadRigXmlFile();
-	}
+	if (!p) return;
+	if (!*p) return;
+
+	progdefaults.XmlRigFilename = p;
+	txtXmlRigFilename->value(fl_filename_name(p));
+	loadRigXmlFile();
 }
 
 void loadRigXmlFile(void)
