@@ -111,7 +111,7 @@ void xmlrpc_rig_set_qsy(long long rfc)
 	wf->rfcarrier(rfc);
 	wf->movetocenter();
 	show_frequency(rfc);
-	LOG_INFO("set qsy: %d", (int)rfc);
+	LOG_VERBOSE("set qsy: %d", (int)rfc);
 }
 
 //======================================================================
@@ -160,7 +160,7 @@ void exec_flrig_ptt() {
 							wait_ptt = true;
 							wait_ptt_timeout = 10;
 							ptt_state = new_ptt;
-							LOG_INFO("ptt %s in %d msec",
+							LOG_VERBOSE("ptt %s in %d msec",
 								ptt_state ? "ON" : "OFF",
 								i*50 + (j + 1)*20);
 							new_ptt = -1;
@@ -211,7 +211,7 @@ void flrig_get_ptt()
 				ptt_state = val;
 				guard_lock flrig_lock(&mutex_flrig_ptt);
 				Fl::awake(xmlrpc_rig_show_ptt, reinterpret_cast<void*>(val) );
-				LOG_INFO("get_ptt: %s", ptt_state ? "ON" : "OFF");
+				LOG_VERBOSE("get_ptt: %s", ptt_state ? "ON" : "OFF");
 			} else if (wait_ptt && (val == ptt_state)) {
 				wait_ptt = false;
 				wait_ptt_timeout = 0;
@@ -267,7 +267,7 @@ void set_flrig_freq(long int fr)
 			wait_freq_timeout = 5;
 			xcvr_freq = fr;
 			Fl::awake(xmlrpc_rig_show_freq, reinterpret_cast<void*>(fr));
-			LOG_INFO("set freq: %d", (int)fr);
+			LOG_VERBOSE("set freq: %d", (int)fr);
 		}
 	} catch (...) {}
 }
@@ -286,7 +286,7 @@ void flrig_get_frequency()
 				xcvr_freq = fr;
 				guard_lock flrig_lock(&mutex_flrig_freq);
 				Fl::awake(xmlrpc_rig_show_freq, reinterpret_cast<void*>(fr));
-				LOG_INFO("get freq: %d", fr);
+				LOG_VERBOSE("get freq: %d", fr);
 			} else if (wait_freq && (fr == xcvr_freq)) {
 				wait_freq = false;
 				wait_freq_timeout = 0;
@@ -338,7 +338,7 @@ void set_flrig_mode(const char *md)
 			bws_posted = false;
 			wait_mode = true;
 			wait_mode_timeout = 5;
-			LOG_INFO("set mode: %s", md);
+			LOG_VERBOSE("set mode: %s", md);
 		}
 	} catch (...) {}
 }
@@ -381,7 +381,7 @@ void flrig_get_mode()
 					xml_USB = true;
 				}
 				Fl::awake(xmlrpc_rig_post_mode, reinterpret_cast<void*>(&md));
-				LOG_INFO("get mode: %s:%s", md.c_str(), xml_USB ? "USB" : "LSB");
+				LOG_VERBOSE("get mode: %s:%s", md.c_str(), xml_USB ? "USB" : "LSB");
 			} else if (wait_mode && posted) {
 				wait_mode = false;
 				wait_mode_timeout = 0;
@@ -440,7 +440,7 @@ void flrig_get_modes()
 				for (int i = 0; i < nargs - 1; i++)
 					debugstr.append(modes_result[i]).append(",");
 				debugstr.append(modes_result[nargs-1]);
-				LOG_INFO("%s", debugstr.c_str());
+				LOG_VERBOSE("%s", debugstr.c_str());
 			}
 		} else {
 			LOG_ERROR("%s", "get modes failed");
@@ -464,7 +464,7 @@ void set_flrig_bw(int bw2, int bw1)
 		val = ival;
 
 		guard_lock flrig_lock(&mutex_flrig);
-		LOG_INFO("set_flrig_bw %04X", ival);
+		LOG_VERBOSE("set_flrig_bw %04X", ival);
 		if (!flrig_client->execute("rig.set_bw", val, result, timeout)) {
 			LOG_ERROR("%s", "rig.set_bw failed");
 			wait_bw = false;
@@ -485,7 +485,7 @@ void xmlrpc_rig_post_bw(void *)
 	if (posted_bw != (std::string)(qso_opBW->value())) {
 		qso_opBW->value(posted_bw.c_str());
 		qso_opBW->redraw();
-		LOG_INFO("Update BW %s", posted_bw.c_str());
+		LOG_VERBOSE("Update BW %s", posted_bw.c_str());
 	}
 }
 
@@ -497,7 +497,7 @@ void xmlrpc_rig_post_bw1(void *)
 	if (posted_bw1 != (std::string)(qso_opBW1->value())) {
 		qso_opBW1->value(posted_bw1.c_str());
 		qso_opBW1->redraw();
-		LOG_INFO("Update combo BW1 %s", posted_bw1.c_str());
+		LOG_VERBOSE("Update combo BW1 %s", posted_bw1.c_str());
 	}
 }
 
@@ -508,7 +508,7 @@ void xmlrpc_rig_post_bw2(void *)
 	if (posted_bw2 != (std::string)(qso_opBW2->value())) {
 		qso_opBW2->value(posted_bw2.c_str());
 		qso_opBW2->redraw();
-		LOG_INFO("Update combo BW2 %s", posted_bw2.c_str());
+		LOG_VERBOSE("Update combo BW2 %s", posted_bw2.c_str());
 	}
 }
 
@@ -584,7 +584,7 @@ void xmlrpc_rig_post_bws(void *)
 				debugstr.append(string(bws_result[1][i])).append(", ");
 			debugstr.append(string(bws_result[1][nargs - 1])).append("\n");
 			debugstr.append(labels1);
-			LOG_INFO("%s", debugstr.c_str());
+			LOG_VERBOSE("%s", debugstr.c_str());
 		}
 
 		try {
@@ -616,7 +616,7 @@ void xmlrpc_rig_post_bws(void *)
 					debugstr.append(string(bws_result[0][i])).append(", ");
 				debugstr.append(string(bws_result[0][nargs - 1])).append("\n");
 				debugstr.append(labels2);
-				LOG_INFO("%s", debugstr.c_str());
+				LOG_VERBOSE("%s", debugstr.c_str());
 			}
 
 		} catch ( XmlRpcException err) {
@@ -648,7 +648,7 @@ void xmlrpc_rig_post_bws(void *)
 				for (int i = 1; i < nargs-1; i++)
 					debugstr.append(string(bws_result[0][i])).append(", ");
 				debugstr.append(string(bws_result[0][nargs - 1]));
-				LOG_INFO("%s", debugstr.c_str());
+				LOG_VERBOSE("%s", debugstr.c_str());
 			}
 
 		} catch (XmlRpcException err) {
@@ -929,7 +929,7 @@ void flrig_connection()
 			string method_str = "\nMethods:\n";
 			for (int i = 0; i < nargs; i++)
 				method_str.append("    ").append(result[i]).append("\n");
-			LOG_INFO("%s", method_str.c_str());
+			LOG_VERBOSE("%s", method_str.c_str());
 			connected_to_flrig = true;
 			poll_interval = 20; // every 200 msec
 			flrig_get_xcvr();
@@ -955,7 +955,7 @@ void connect_to_flrig()
 		flrig_client = new XmlRpcClient(
 				progdefaults.flrig_ip_address.c_str(),
 				atol(progdefaults.flrig_ip_port.c_str()));
-		LOG_INFO("created flrig xmlrpc client  %s, %ld",
+		LOG_VERBOSE("created flrig xmlrpc client  %s, %ld",
 				progdefaults.flrig_ip_address.c_str(),
 				atol(progdefaults.flrig_ip_port.c_str()));
 		flrig_connection();
@@ -971,7 +971,7 @@ void * flrig_thread_loop(void *d)
 	while(run_flrig_thread) {
 		for (int i = 0; i < poll_interval; i++) {
 			if (!run_flrig_thread) {
-//				LOG_INFO("Exiting thread - 1");
+//				LOG_VERBOSE("Exiting thread - 1");
 				return NULL;
 			}
 			MilliSleep(10);
@@ -1003,7 +1003,7 @@ void * flrig_thread_loop(void *d)
 			}
 		}
 	}
-//	LOG_INFO("Exiting thread - 2");
+//	LOG_VERBOSE("Exiting thread - 2");
 	return NULL;
 }
 
@@ -1020,13 +1020,13 @@ void FLRIG_start_flrig_thread()
 void stop_flrig_thread()
 {
 	if (!flrig_client) return;
-	LOG_INFO("%s", "stopping flrig thread");
+	LOG_VERBOSE("%s", "stopping flrig thread");
 	flrig_client->close();
 	pthread_mutex_lock(&mutex_flrig);
 		run_flrig_thread = false;
 	pthread_mutex_unlock(&mutex_flrig);
 	pthread_join(*flrig_thread, NULL);
-	LOG_INFO("%s", "flrig thread closed");
+	LOG_VERBOSE("%s", "flrig thread closed");
 }
 
 void reconnect_to_flrig()
