@@ -23,12 +23,14 @@
 #include <config.h>
 
 #include <list>
-#ifdef __clang__
-#include <unordered_map>
+
+#if HAVE_STD_HASH
+#	include <unordered_map>
+#elif HAVE_STD_TR1_HASH
+#	include <tr1/unordered_map>
 #else
-#include <tr1/unordered_map>
+#	error "No std::hash or std::tr1::hash support"
 #endif
-#include <functional>
 
 #include "trx.h"
 #include "globals.h"
@@ -64,14 +66,15 @@ struct fre_comp : std::unary_function<const fre_t*, bool>
 };
 
 typedef list<callback_t*> callback_p_list_t;
-#ifdef __clang__
-typedef std::unordered_map<fre_t*, callback_p_list_t, fre_hash, fre_comp> rcblist_t;
-static std::unordered_map<int, string> buffers;
-#else
-typedef tr1::unordered_map<fre_t*, callback_p_list_t, fre_hash, fre_comp> rcblist_t;
 
-static tr1::unordered_map<int, string> buffers;
+#if HAVE_STD_HASH
+	typedef std::unordered_map<fre_t*, callback_p_list_t, fre_hash, fre_comp> rcblist_t;
+	static std::unordered_map<int, string> buffers;
+#elif HAVE_STD_TR1_HASH
+	typedef tr1::unordered_map<fre_t*, callback_p_list_t, fre_hash, fre_comp> rcblist_t;
+	static tr1::unordered_map<int, string> buffers;
 #endif
+
 static cblist_t cblist;
 static rcblist_t rcblist;
 
