@@ -164,6 +164,7 @@ modem *contestia_modem = 0;
 modem *rtty_modem = 0;
 modem *pkt_modem = 0;
 
+modem *thormicro_modem = 0;
 modem *thor4_modem = 0;
 modem *thor5_modem = 0;
 modem *thor8_modem = 0;
@@ -179,6 +180,7 @@ modem *fsq_modem = 0;
 
 modem *ifkp_modem = 0;
 
+modem *dominoexmicro_modem = 0;
 modem *dominoex4_modem = 0;
 modem *dominoex5_modem = 0;
 modem *dominoex8_modem = 0;
