@@ -33,7 +33,6 @@
 #include "ascii.h"
 
 #define null_bw 1
-#define null_symbol_len 256
 
 // a NULLMODEM and will be instantiated before the dynamic member wf, 
 // digiscope, and fl_digi_main, the main dialog
@@ -89,13 +88,19 @@ int NULLMODEM::rx_process(const double *buf, int len)
 int NULLMODEM::tx_process()
 {
 	MilliSleep(10);
-	if (!fl_digi_main) return 0;
+	if (!fl_digi_main) {
+		return 0;
+	}
 
-	if ( get_tx_char() == GET_TX_CHAR_ETX || stopflag) {
+	int c = get_tx_char();
+	if (c == GET_TX_CHAR_ETX) {
+		stopflag = false;
+		return -1;
+	}
+	if ( stopflag ) {
 		stopflag = false;
 		return -1;
 	}
-	ModulateXmtr(outbuf, null_symbol_len);
 	return 0;
 }
 
